Package: prodest
Type: Package
Title: Production Function Estimation
Version: 1.0.1
Author: Gabriele Rovigatti [aut,cre]
Maintainer: Gabriele Rovigatti <gabriele.rovigatti@gmail.com>
Description: Implements the methods proposed by Olley, G.S. and Pakes, A. (1996) <doi:10.2307/2171831>, Levinsohn, J. and Petrin, A. (2003) <doi:10.1111/1467-937X.00246>, Ackerberg, D.A. and Caves, K. and Frazer, G. (2015) <doi:10.3982/ECTA13408> and Wooldridge, J.M. (2009) <doi:10.1016/j.econlet.2009.04.026> for structural productivity estimation .
License: GPL-3
BugReports: https://github.com/GabrieleRovigatti/prodest/issues
URL: https://github.com/GabrieleRovigatti/prodest/tree/master/prodest
LazyData: TRUE
Depends: R (>= 2.10), dplyr, parallel, Matrix, methods
Imports: Rsolnp, DEoptim, AER
Suggests: testthat
Repository: CRAN
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2018-06-19 13:10:04 UTC; gabriele
Date/Publication: 2018-06-19 13:21:32 UTC
Built: R 4.6.0; ; 2026-05-01 02:14:13 UTC; unix

Package: prodest
Type: Package
Title: Production Function Estimation
Version: 1.0.2
Authors@R: person(given = "Gabriele",
                  family = "Rovigatti",
                  role = c("aut", "cre"),
                  email = "gabriele.rovigatti@gmail.com")
Description: Implements the methods proposed by Olley, G.S. and Pakes, A. (1996) <doi:10.2307/2171831>, Levinsohn, J. and Petrin, A. (2003) <doi:10.1111/1467-937X.00246>, Ackerberg, D.A. and Caves, K. and Frazer, G. (2015) <doi:10.3982/ECTA13408> and Wooldridge, J.M. (2009) <doi:10.1016/j.econlet.2009.04.026> for structural productivity estimation.
License: GPL-3
BugReports: https://github.com/GabrieleRovigatti/prodest/issues
URL: https://github.com/GabrieleRovigatti/prodest
LazyData: TRUE
Depends: R (>= 2.10), dplyr, parallel, Matrix, methods
Imports: Rsolnp, DEoptim, AER
Suggests: testthat
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2026-09-01 14:25:47 UTC; m031421
Author: Gabriele Rovigatti [aut, cre]
Maintainer: Gabriele Rovigatti <gabriele.rovigatti@gmail.com>
Repository: CRAN
Date/Publication: 2026-09-01 15:20:02 UTC

#exportPattern("^[[:alpha:]]+")
import("methods")
import("DEoptim")
importFrom("Rsolnp", "solnp")
import("dplyr")
importFrom("AER", "ivreg")
importFrom("parallel", "clusterEvalQ", "parLapply")
importFrom("Matrix", "bdiag")
importFrom("stats", "fitted", "lm", "model.frame", "model.matrix",
     "na.exclude", "optim", "resid", "rnorm", "runif", "sd",
     "var", "binomial", "glm", "lm.fit", "predict")
exportClasses("prod")
exportMethods("coef","omega","FSres","summary","show")
export("coef","omega","FSres")
export("prodestOP","prodestLP","prodestACF","prodestWRDG","prodestWRDG_GMM","panelSim","prodestROB")
export("printProd")
export("gOPLP","gACF","finalOPLP","finalACF")
export("checkM", "checkMD","lagPanel","block.boot.resample","weightM","withinvar")

# prodest
Production Function Estimation in R

`prodest` ([Rovigatti, 2017]) implements most of the methods for production function estimation. The prodest package provides 
functions to simulate panel data,
estimate the productivity, state and free variables parameters. Full description of the models, their issues and characteristics
is available in [Mollisi et al., 2017](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=2916753).

The latest stable version of `prodest` is available at 'https://CRAN.R-project.org/package=prodest'.

The latest development version of `prodest` is available at 'https://github.com/GabrieleRovigatti/prodest'.

Please cite `prodest` in publications:

Rovigatti, G. (2026).  
_prodest: Production Function Estimation_.  
R package version 1.0.2.

Rovigatti, G. (2017).  
Production Function Estimation in R: The prodest Package.  
_Journal of Open Source Software_, 2(18), 371.  
https://doi.org/10.21105/joss.00371  
 

############ ROBINSON/WOOLDRIDGE - IV REGRESSION ###############

# function to estimate Wooldridge #
prodestROB <- function(Y, fX, sX, pX, idvar, timevar, cX = NULL){
  Start = Sys.time() # start tracking time
  Y <- checkM(Y) # change all input to matrix
  fX <- checkM(fX)
  sX <- checkM(sX)
  pX <- checkM(pX)
  idvar <- checkM(idvar)
  timevar <- checkM(timevar)
  opt = 'optim' # this is going to be changed once the routine will work with DEoptim and solnp
  snum <- ncol(sX) # find the number of input variables
  fnum <- ncol(fX)
  if (!is.null(cX)) {cX <- checkM(cX); cnum <- ncol(cX)} else {cnum <- 0} # if is there any control, take it into account, else fix the number of controls to 0
  lag.fX = fX # generate fX lags
  for (i in 1:fnum) {
    lag.fX[, i] = lagPanel(fX[, i], idvar = idvar, timevar = timevar)
  }
  polyframe <- data.frame(sX,pX) # vars to be used in polynomial approximation
  regvars <- cbind(model.matrix( ~.^2-1, data = polyframe), sX^2, pX^2) # generate a polynomial of the desired level
  lagregvars <- regvars
  for (i in 1:dim(regvars)[2]) {
    lagregvars[, i] <- lagPanel(idvar = idvar, timevar = timevar, regvars[ ,i])
  }
  data <- model.frame(Y ~ fX + sX + lag.fX + regvars + lagregvars + idvar + timevar) # data.frame of usable observations --> regvars

  iv.out <- ivreg(Y ~ fX + sX + lagregvars | lag.fX + sX + lagregvars, data = data)
  res.names <- c(colnames(fX, do.NULL = FALSE, prefix = 'fX'),
                 colnames(sX, do.NULL = FALSE, prefix = 'sX') ) # generate the list of names for results

  betapar <- iv.out$coefficients[2: (snum + fnum + cnum + 1)]
  betase <- coef(summary(iv.out))[, 2][2: (snum + fnum + cnum + 1)]
  names(betapar) <- res.names # change results' names
  names(betase) <- res.names # change results' names
  elapsed.time = Sys.time() - Start # total running time
  out <- new("prod",
             Model = list(method = 'ROB-IV', FSbetas = NULL, boot.repetitions = NA, elapsed.time = elapsed.time, theta0 = NA,
                          opt = NA, opt.outcome = NULL, nCores = 1),
             Data = list(Y = Y, free = fX, state = sX, proxy = pX, control = cX, idvar = idvar, timevar = timevar,
                         FSresiduals = NULL),
             Estimates = list(pars = betapar, std.errors = betase))
  return(out)
}
## end of prodest Robinson-Wooldridge ##

library(prodest)

## Regression test for vector and matrix inputs to lagPanel().
id <- c(1, 1, 2, 2)
time <- c(1, 2, 1, 2)
x <- c(10, 20, 30, 40)
expected <- c(NA_real_, 10, NA_real_, 30)

stopifnot(isTRUE(all.equal(
  lagPanel(id, time, x),
  expected,
  check.attributes = FALSE
)))

stopifnot(isTRUE(all.equal(
  lagPanel(x, idvar = id, timevar = time),
  expected,
  check.attributes = FALSE
)))

x_matrix <- cbind(a = x, b = x + 100)
lagged_matrix <- lagPanel(id, time, x_matrix)
stopifnot(
  is.matrix(lagged_matrix),
  identical(dim(lagged_matrix), c(4L, 2L)),
  identical(colnames(lagged_matrix), c("la", "lb")),
  isTRUE(all.equal(lagged_matrix[, 1], expected, check.attributes = FALSE)),
  isTRUE(all.equal(lagged_matrix[, 2], expected + 100, check.attributes = FALSE))
)

## Regression test for the panelSim() vector-recycling warning.
set.seed(123)
warnings_seen <- character()
simulated <- withCallingHandlers(
  panelSim(N = 5, T = 20),
  warning = function(w) {
    warnings_seen <<- c(warnings_seen, conditionMessage(w))
    invokeRestart("muffleWarning")
  }
)
stopifnot(
  length(warnings_seen) == 0L,
  nrow(simulated) == 10L,
  ncol(simulated) == 9L,
  identical(
    names(simulated),
    c("idvar", "timevar", "Y", "sX", "fX", "pX1", "pX2", "pX3", "pX4")
  )
)

## Regression tests for character panel identifiers (GitHub issue #20).
character_id <- c("firm-a", "firm-a", "firm-b", "firm-b", "firm-b", "firm-c")
character_time <- c(1, 2, 1, 2, 3, 1)
character_value <- seq_along(character_id)
expected_character_lag <- c(NA_real_, 1, NA_real_, 3, 4, NA_real_)
stopifnot(isTRUE(all.equal(
  lagPanel(character_id, character_time, character_value),
  expected_character_lag,
  check.attributes = FALSE
)))

panel_id_code <- getFromNamespace(".panel_id_code", "prodest")
encoded_id <- panel_id_code(character_id)
stopifnot(
  is.matrix(encoded_id),
  identical(dim(encoded_id), c(6L, 1L)),
  identical(as.integer(encoded_id), c(1L, 1L, 2L, 2L, 2L, 3L))
)

set.seed(321)
bootstrap_indices <- block.boot.resample(character_id, R = 4)
stopifnot(length(bootstrap_indices) == 4L)
for (indices in bootstrap_indices) {
  stopifnot(
    is.matrix(indices),
    ncol(indices) == 1L,
    all(indices[, 1] %in% seq_along(character_id)),
    !is.null(rownames(indices))
  )
  sampled_panels <- split(as.integer(indices[, 1]), rownames(indices))
  stopifnot(length(sampled_panels) == length(unique(character_id)))
  for (rows in sampled_panels) {
    source_id <- character_id[rows[1]]
    stopifnot(
      length(unique(character_id[rows])) == 1L,
      identical(unname(rows), which(character_id == source_id))
    )
  }
}

## Regression test for ACF controls (GitHub issue #11).
prepare_acf_data <- getFromNamespace(".prepare_acf_data", "prodest")
acf_id <- rep(c("a", "b", "c"), each = 5)
acf_time <- rep(seq_len(5), 3)
acf_n <- length(acf_id)
acf_fX <- matrix(seq(0.2, 1.6, length.out = acf_n), ncol = 1,
                 dimnames = list(NULL, "labour"))
acf_sX <- matrix(seq(1.1, 2.5, length.out = acf_n), ncol = 1,
                 dimnames = list(NULL, "capital"))
acf_pX <- matrix(seq(0.7, 1.4, length.out = acf_n), ncol = 1,
                 dimnames = list(NULL, "materials"))
acf_cX <- matrix(seq(-0.3, 0.4, length.out = acf_n), ncol = 1,
                 dimnames = list(NULL, "control"))
acf_Y <- matrix(1 + 0.4 * acf_fX + 0.3 * acf_sX + 0.2 * acf_cX,
                ncol = 1)
acf_data <- prepare_acf_data(
  Y = acf_Y, fX = acf_fX, sX = acf_sX, pX = acf_pX,
  idvar = acf_id, timevar = acf_time, cX = acf_cX
)
stopifnot(
  is.matrix(acf_data),
  storage.mode(acf_data) == "double",
  sum(grepl("^Z", colnames(acf_data))) == 3L,
  sum(grepl("^Xt", colnames(acf_data))) == 3L,
  sum(grepl("^lX", colnames(acf_data))) == 3L,
  identical(
    as.integer(acf_data[, "idvar"]),
    rep(1:3, each = 5)
  )
)
acf_regvars <- acf_data[, grepl("^regvars", colnames(acf_data)), drop = FALSE]
stopifnot(
  isTRUE(all.equal(acf_regvars[, 1], acf_fX[, 1], check.attributes = FALSE)),
  isTRUE(all.equal(acf_regvars[, 2], acf_sX[, 1], check.attributes = FALSE)),
  isTRUE(all.equal(acf_regvars[, 3], acf_cX[, 1], check.attributes = FALSE))
)

## The GMM criterion must accept one coefficient per free, state and control
## regressor without a non-conformable-matrix error.
criterion_n <- 30L
criterion_index <- seq_len(criterion_n)
criterion_theta <- c(0.2, 0.3, 0.1)
criterion_X <- cbind(
  criterion_index / criterion_n,
  sin(criterion_index / 3),
  cos(criterion_index / 5)
)
criterion_lX <- cbind(
  (criterion_index - 1) / criterion_n,
  sin((criterion_index - 1) / 3),
  cos((criterion_index - 1) / 5)
)
criterion_omega_lag <- seq(-1, 1, length.out = criterion_n)
criterion_lag_phi <- criterion_lX %*% criterion_theta + criterion_omega_lag
criterion_omega <- 0.4 + 0.7 * criterion_omega_lag +
  0.2 * criterion_omega_lag^2 + 0.05 * criterion_index / criterion_n
criterion_phi <- criterion_X %*% criterion_theta + criterion_omega
criterion_Z <- cbind(
  criterion_index / criterion_n,
  sin(criterion_index / 4),
  cos(criterion_index / 6)
)
criterion_W <- solve(crossprod(criterion_Z)) / criterion_n
criterion_value <- gACF(
  criterion_theta, mZ = criterion_Z, mW = criterion_W,
  mX = criterion_X, mlX = criterion_lX,
  vphi = criterion_phi, vlag.phi = criterion_lag_phi
)
stopifnot(
  is.matrix(criterion_value),
  identical(dim(criterion_value), c(1L, 1L)),
  is.finite(criterion_value[1, 1])
)

## Regression test for the corrected packaged Chilean data (GitHub issue #22).
data("chilean", package = "prodest")
stopifnot(
  nrow(chilean) == 2544L,
  identical(
    names(chilean),
    c("Y", "sX", "fX1", "fX2", "pX", "inv", "idvar", "timevar")
  ),
  !"cX" %in% names(chilean),
  identical(range(chilean$timevar), c(1996, 2006)),
  !isTRUE(all.equal(chilean$pX, chilean$inv, check.attributes = FALSE))
)

############ WOOLDRIDGE ###############

# function to estimate Wooldridge #
prodestWRDG_GMM <- function(Y, fX, sX, pX, idvar, timevar, cX = NULL, tol = 1e-100){
  Start = Sys.time() # start tracking time
  Y <- checkM(Y) # change all input to matrix
  fX <- checkM(fX)
  sX <- checkM(sX)
  pX <- checkM(pX)
  idvar <- checkM(idvar)
  timevar <- checkM(timevar)
  snum <- ncol(sX) # find the number of input variables
  fnum <- ncol(fX)
  if (!is.null(cX)) {cX <- checkM(cX); cnum <- ncol(cX)} else {cnum <- 0} # if is there any control, take it into account, else fix the number of controls to 0
  lag.fX = fX # generate fX lags
  for (i in 1:fnum) {
    lag.fX[, i] = lagPanel(fX[, i], idvar = idvar, timevar = timevar)
  }
  polyframe <- data.frame(sX,pX) # vars to be used in polynomial approximation
  regvars <- cbind(model.matrix( ~.^2-1, data = polyframe),sX^2,pX^2) # generate a polynomial of the desired level
  lagregvars <- regvars
  for (i in 1:dim(regvars)[2]) {
    lagregvars[, i] <- lagPanel(idvar = idvar, timevar = timevar, regvars[ ,i])
  }
  data <- model.frame(Y ~ fX + sX + lag.fX + regvars + lagregvars + idvar + timevar) # data.frame of usable observations --> regvars

  Y <- data$Y; dY = c(data$Y, data$Y); X1 = cbind(1, data$fX, data$regvars)
  X2 = cbind(1, data$fX, data$sX, 1, data$lagregvars)
  Z1 = cbind(1, data$fX, data$regvars)
  Z2 = cbind(1, data$lag.fX, data$sX, data$lagregvars) # define the data and the matrices
  numR = 1 + fnum + snum + cnum # number of restricted columns
  numU1 <- ncol(X1) - numR # number of unrestricted cols of X1
  numU2 <- ncol(X2) - numR # number of unrestricted cols of X2
  N <- nrow(X1)

  dX <- rbind( cbind( X1, matrix(0, N, numU2 ) ),
               cbind( X2[, 1 : numR], matrix(0 , N, numU1), X2[,(numR + 1) : ncol(X2)]) ) # generate a "quasi-block" matrix with common columns NON-BLOCK
  Z <- as.matrix(bdiag(Z1, Z2))
  W <- solve((t(Z) %*% Z)) * diag(ncol(Z)) # unadjusted, independent

  betas.1st <- solve(t(dX) %*% Z %*% W %*% t(Z) %*% dX, tol = tol) %*%
                  t(dX) %*% Z %*% W %*% t(Z) %*% dY # 1st stage GMM parameters
  W.star <- weightM(Y = Y, X1 = X1, X2 = X2, Z1 = Z1, Z2 = Z2,
                    betas = betas.1st, numR = (fnum + snum + cnum + 1)) # compute optimal weighting matrix
  betas.2nd <- solve(t(dX) %*% Z %*% W.star %*% t(Z) %*% dX, tol = tol) %*%
                  t(dX) %*% Z %*% W.star %*% t(Z) %*% dY # 2nd step
  st.errors <- weightM(Y = Y, X1 = X1, X2 = X2, Z1 = Z1, Z2 = Z2,
                       betas = betas.2nd, numR = (fnum + snum + cnum + 1), SE = TRUE) # compute standard errors

  res.names <- c(colnames(fX, do.NULL = FALSE, prefix = 'fX'),
                 colnames(sX, do.NULL = FALSE, prefix = 'sX') ) # generate the list of names for results
  if (!is.null(cX)) {res.names <- c(res.names, colnames(cX, do.NULL = FALSE, prefix = 'cX'))} # add cX names to the results' names
  betapar <- betas.2nd[2: (snum + fnum + cnum + 1)]
  betase <- st.errors[2: (snum + fnum + cnum + 1)]
  names(betapar) <- res.names # change results' names
  names(betase) <- res.names # change results' names
  elapsed.time = Sys.time() - Start # total running time
  out <- new("prod",
             Model = list(method = 'WRDG.GMM', boot.repetitions = NA, elapsed.time = elapsed.time, theta0 = NA,
                          opt = NA, opt.outcome = NULL),
             Data = list(Y = Y, free = fX, state = sX, proxy = pX, control = cX, idvar = idvar, timevar = timevar),
             Estimates = list(pars = betapar, std.errors = betase))
  return(out)
}
# end of Wooldridge #

# set the "prod" object class #
setClass("prod", representation(Model = "list", Data = "list", Estimates = "list"))
# end of object classs setting #

# coefficients #
.getCoef <- function(object, do.list = FALSE){
  object@Estimates$pars
}
setMethod("coef", "prod", .getCoef ) # , iBurnPeriod = NULL
# end coefficients #

# Report the Omega estimate #
setGeneric("omega", function(object) standardGeneric("omega")) # set the generic "omega" function for "prod" objects
setMethod("omega", "prod", function(object){ # , iBurnPeriod = NULL
  Omega <- object@Data$Y - cbind(object@Data$free, object@Data$state, object@Data$control) %*% object@Estimates$pars
  return(Omega)
})
# end of omega #

# First stage residuals #
setGeneric("FSres", function(object) standardGeneric("FSres")) # set the generic "FSres" function for "prod" objects
setMethod("FSres", "prod", function(object) object@Data$FSresiduals) # , iBurnPeriod = NULL
# end FS residuals #

# Show method #
setMethod("show", "prod", function(object){

  Pars = object@Estimates$pars
  nPars = names(Pars)
  st.err = object@Estimates$std.errors
  Method = object@Model$method

  cat("\n-------------------------------------------------------")
  cat("\n-            Production Function Estimation           -")
  cat("\n-------------------------------------------------------")
  cat(paste("\n                   Method:   ", Method, "             "))
  cat("\n-------------------------------------------------------")
  cat(paste("\n                      ", paste("  ", nPars, collapse = "    ", " ", sep = "")))
  cat(paste("\nEstimated Parameters: ", paste(" ", round(Pars, digits = 3), collapse = "   ", " ", sep = "")))
  cat(paste("\n                      ", paste("(", round(st.err, digits = 3), collapse = "   ", ")", sep = "")))
  cat("\n-------------------------------------------------------")
})
# end of show method

# Summary method #
setMethod("summary", "prod", function(object, ...){

  Pars = object@Estimates$pars
  namePars = names(Pars)
  st.err = object@Estimates$std.errors
  Method = object@Model$method
  N = nrow(object@Data$Y)
  Time = object@Model$elapsed.time
  nCores = object@Model$nCores
  opt = object@Model$opt
  R = object@Model$boot.repetitions
  theta0 = object@Model$theta0

  cat("\n-------------------------------------------------------------")
  cat("\n-               Production Function Estimation              -")
  cat("\n-------------------------------------------------------------")
  cat(paste("\n                   Method :   ", Method, "             "))
  cat("\n-------------------------------------------------------------")
  cat(paste("\n                            ", paste("  ", namePars, collapse = "    ", " ", sep = "")))
  cat(paste("\nEstimated Parameters      : ", paste(" ", round(Pars, digits = 3), collapse = "   ", " ", sep = "")))
  cat(paste("\n                            ", paste("(", round(st.err, digits = 3), collapse = "   ", ")", sep = "")))
  cat("\n-------------------------------------------------------------")
  cat(paste("\nN                         :  ", N, sep = "") )
  cat("\n-------------------------------------------------------------")
  if (!(Method == 'WRDG')){
    cat(paste("\nBootstrap repetitions     : ", R))
    FS.betas = object@Model$FSbetas[2 : (1 + length(Pars))]
    cat(paste("\n1st Stage Parameters      :", paste(" ", round(FS.betas, digits = 3), collapse = "   ", " ", sep = "")))
    cat(paste("\nOptimizer                 : ", opt))
    if (!is.null(theta0)){
      cat(paste("\n2nd Stage Start Points    :", paste(" ", theta0, collapse = "   ", " ", sep = "")))
    }
  }
  cat("\n-------------------------------------------------------------")
  cat(paste("\nElapsed Time              :  ", round(as.double(Time, units = 'mins'), digits = 2), " mins", sep = "") )
  if (!is.null(nCores)){
    cat(paste("\n# Cores                 : ", nCores, sep = "") )
  }
  cat("\n-------------------------------------------------------------")
})
# end of summary method

############ OLLEY-PAKES and LEVINSOHN-PETRIN ###############

# function to estimate OP and LP model #
prodestOP <- function(Y, fX, sX, pX, idvar, timevar, R = 20, cX = NULL, opt = 'optim',
                      theta0 = NULL, cluster = NULL, tol = 1e-100, exit = FALSE){
  Start = Sys.time() # start tracking time
  Y <- checkM(Y) # change all input to matrix
  fX <- checkM(fX)
  sX <- checkM(sX)
  pX <- checkM(pX)
  idvar <- checkM(idvar)
  idvar.original <- idvar
  idvar.internal <- .panel_id_code(idvar)
  timevar <- checkM(timevar)
  snum <- ncol(sX) # find the number of input variables
  fnum <- ncol(fX)
  if (!is.null(cX)) {cX <- checkM(cX); cnum <- ncol(cX)} else {cnum <- 0} # if is there any control, take it into account, else fix the number of controls to 0
  polyframe <- data.frame(cbind(sX,pX)) # vars to be used in polynomial approximation
  mod <- model.matrix( ~ .^2 - 1, data = polyframe) # generate the polynomial elements - this drops NAs
  mod <- mod[match(rownames(polyframe), rownames(mod)),] # replace NAs if there was any
  ### WORK ON ATTRITION - generate fitted survival probabilities ###
  if (exit[1] == TRUE){ # if 'exit' is either true, or a matrix/vector of entry and exit
    att = TRUE
    if (is.logical(exit)){ # if exit is true, generate the vector of exits
      exitdata = data.frame(idvar = idvar, timevar = timevar)
      maxdate <- max(exitdata$timevar)
      exit <- as.matrix(do.call(rbind, lapply(split(exitdata, list(exitdata$idvar)),
                                              function(x){
                                                maxid = max(x$timevar)
                                                x$ans <- x$timevar == maxid & maxid != maxdate # generate the dummy for last seen year not being the last year in the sample
                                                cbind(x$ans) } )))
    } else {
      exit <- checkMD(exit) # change the input data to matrix
    }
    if ( (mean(exit) == 1) | (mean(exit) == 0) ){
      stop("No ID appears to exit the sample. Check the exit variable or run the model without exit = TRUE")
    } else {
      lagProbitvars <- cbind(mod, sX^2, pX^2) # generate the lagged interaction variables
      for (i in 1:dim(lagProbitvars)[2]){
        lagProbitvars[, i] <- lagPanel(lagProbitvars[, i], idvar = idvar, timevar = timevar)
      }
      probreg <- glm(exit ~ lagProbitvars, family = binomial(link = 'probit')) # run the probit regression
      Pr.hat <- predict(probreg, newdata = as.data.frame(lagProbitvars), type = 'response') # predict the fitted exit probability
    }
  } else {
    att = FALSE
    Pr.hat <- matrix(0, nrow = nrow(mod), ncol = 1) # generate fitted probability as all 0s
  }
  ### END OF ATTRITION ###
  regvars <- cbind(fX, cX, mod, sX^2, pX^2) # generate a polynomial of the desired level
  lag.sX = sX # generate sX lags
  for (i in 1:snum) {
    lag.sX[, i] = lagPanel(sX[, i], idvar = idvar, timevar = timevar)
  }
  data <- suppressWarnings(as.matrix(data.frame(state = sX, lag.sX = lag.sX, free = fX, Y = Y, idvar = idvar.internal, # generate the matrix of data
                                                timevar = timevar, regvars = regvars, Pr.hat = Pr.hat)))
  if (!is.null(cX)) {
    data <- suppressWarnings(as.matrix(data.frame(data, cX = cX)))
  }
  betas <- finalOPLP(ind = TRUE, data = data, fnum = fnum, snum = snum, cnum = cnum, opt = opt,
                     theta0 = theta0, boot = FALSE, tol = tol, att = att) # generate the list of estimated betas
  boot.indices <- block.boot.resample(idvar, R) # generate a list: every element has different length (same IDs, different time occasions) and is a vector of new indices, whose rownames are the new IDs
  if (is.null(cluster)){
    nCores = NULL
    boot.betas <- matrix(unlist(lapply(boot.indices, finalOPLP, data = data, fnum = fnum, snum = snum, cnum = cnum,
                                       opt = opt, theta0 = theta0, boot = TRUE, tol = tol, att = att)), ncol = fnum+snum+cnum, byrow = TRUE) # use the indices and pass them to the final function (reshape the data)
  } else { # set up the cluster: send the lag Panel and the package to clusters
    nCores = length(cluster)
    clusterEvalQ(cl = cluster, library(prodest))
    boot.betas <- matrix(unlist(parLapply(cl = cluster, boot.indices, finalOPLP, data = data,
                                          fnum = fnum, snum = snum, cnum = cnum,
                                          opt = opt, theta0 = theta0, boot = TRUE, tol = tol, att = att)), ncol = fnum+snum+cnum,
                         byrow = TRUE) # use the indices and pass them to the final function (reshape the data)
  }
  boot.errors <- apply(boot.betas,2,sd,na.rm=TRUE) # calculate standard deviations
  res.names <- c(colnames(fX, do.NULL = FALSE, prefix = 'fX'),
                 colnames(sX, do.NULL = FALSE, prefix = 'sX') ) # generate the list of names for results
  if (!is.null(cX)) {res.names <- c(res.names,colnames(cX, do.NULL = FALSE, prefix = 'cX'))}
  names(betas$betas) <- res.names # change results' names
  names(betas$FSbetas)[2 : length(betas$FSbetas)] <- c(res.names, rep(" ", (length(betas$FSbetas) - length(res.names) - 1))) # change 1st stage results' names
  names(boot.errors) <- res.names # change std.errors' names
  elapsedTime = Sys.time() - Start # total running time
  out <- new("prod",
             Model = list(method = 'OP', FSbetas = betas$FSbetas, boot.repetitions = R, elapsed.time = elapsedTime,
                          theta0 = theta0 , opt = opt, opt.outcome = betas$opt.outcome, nCores = nCores),
             Data = list(Y = Y, free = fX, state = sX, proxy = pX, control = cX, idvar = idvar.original, timevar = timevar,
                         FSresiduals = betas$FSresiduals),
             Estimates = list(pars = betas$betas, std.errors = boot.errors))
  return(out)
}
# end of prodestOP #

# function to estimate OP and LP model #
prodestLP <- function(Y, fX, sX, pX, idvar, timevar, R = 20, cX = NULL, opt = 'optim',
                      theta0 = NULL, cluster = NULL, tol = 1e-100, exit = FALSE){
  Start = Sys.time() # start tracking time
  Y <- checkM(Y) # change all input to matrix
  fX <- checkM(fX)
  sX <- checkM(sX)
  pX <- checkM(pX)
  idvar <- checkM(idvar)
  idvar.original <- idvar
  idvar.internal <- .panel_id_code(idvar)
  timevar <- checkM(timevar)
  snum <- ncol(sX) # find the number of input variables
  fnum <- ncol(fX)
  if (!is.null(cX)) {cX <- checkM(cX); cnum <- ncol(cX)} else {cnum <- 0} # if is there any control, take it into account, else fix the number of controls to 0
  polyframe <- data.frame(cbind(sX,pX)) # vars to be used in polynomial approximation
  mod <- model.matrix( ~.^2-1, data = polyframe) # generate the polynomial elements - this drops NAs
  mod <- mod[match(rownames(polyframe),rownames(mod)),] # replace NAs if there was any
  ### WORK ON ATTRITION - generate fitted survival probabilities ###
  if (exit[1] == TRUE){ # if 'exit' is either true, or a matrix/vector of entry and exit
    att = TRUE
    if (is.logical(exit)){ # if exit is true, generate the vector of exits
      exitdata = data.frame(idvar = idvar, timevar = timevar)
      maxdate <- max(exitdata$timevar)
      exit <- as.matrix(do.call(rbind, lapply(split(exitdata, list(exitdata$idvar)),
                                              function(x){
                                                maxid = max(x$timevar)
                                                x$ans <- x$timevar == maxid & maxid != maxdate # generate the dummy for last seen year not being the last year in the sample
                                                cbind(x$ans) } )))
    } else {
      exit <- checkMD(exit) # change the input data to matrix
    }
    if ( (mean(exit) == 1) | (mean(exit) == 0) ){
      stop("No ID appears to exit the sample. Check the exit variable or run the model without exit = TRUE")
    } else {
      lagProbitvars <- cbind(mod, sX^2, pX^2) # generate the lagged interaction variables
      for (i in 1:dim(lagProbitvars)[2]){
        lagProbitvars[, i] <- lagPanel(lagProbitvars[, i], idvar = idvar, timevar = timevar)
      }
      probreg <- glm(exit ~ lagProbitvars, family = binomial(link = 'probit')) # run the probit regression
      Pr.hat <- predict(probreg, newdata = as.data.frame(lagProbitvars), type = 'response') # predict the fitted exit probability
    }
  } else {
    att = FALSE
    Pr.hat <- matrix(0, nrow = nrow(mod), ncol = 1) # generate fitted probability as all 0s
  }
  ### END OF ATTRITION ###
  regvars <- cbind(fX, cX, mod, sX^2, pX^2) # generate a polynomial of the desired level
  lag.sX = sX # generate sX lags
  for (i in 1:snum) {
    lag.sX[, i] = lagPanel(sX[, i], idvar = idvar, timevar = timevar)
  }
  data <- suppressWarnings(as.matrix(data.frame(state = sX, lag.sX = lag.sX, free = fX, Y = Y, idvar = idvar.internal, # generate the matrix of data
                                                timevar = timevar, regvars = regvars, Pr.hat = Pr.hat)))
  if (!is.null(cX)) {
    data <- suppressWarnings(as.matrix(data.frame(data, cX = cX)))
  }
  betas <- finalOPLP(ind = TRUE, data = data, fnum = fnum, snum = snum, cnum = cnum, opt = opt,
                     theta0 = theta0, boot = FALSE, tol = tol, att = att) # generate the list of estimated betas
  boot.indices <- block.boot.resample(idvar, R) # generate a list: every element has different length (same IDs, different time occasions) and is a vector of new indices, whose rownames are the new IDs
  if (is.null(cluster)){
    nCores = NULL
    boot.betas <- matrix(NA, R, (fnum + snum + cnum))
    for (i in 1:R){
      boot.betas[i,] <- finalOPLP(ind = boot.indices[[i]], data = data, fnum = fnum, snum = snum, cnum = cnum,
                                  opt = opt, theta0 = theta0, boot = TRUE, tol = tol, att = att)
    }
  } else { # set up the cluster: send the lag Panel and the data.table libraries to clusters
    nCores = length(cluster)
    clusterEvalQ(cl = cluster, library(prodest))
    boot.betas <- matrix(unlist(parLapply(cl = cluster, boot.indices, finalOPLP, data = data,
                                          fnum = fnum, snum = snum, cnum = cnum,
                                          opt = opt, theta0 = theta0, boot = TRUE, tol = tol, att = att)), ncol = fnum + snum + cnum,
                         byrow = TRUE) # use the indices and pass them to the final function (reshape the data)
  }
  boot.errors <- apply(boot.betas, 2, sd, na.rm = TRUE) # calculate standard deviations
  res.names <- c(colnames(fX, do.NULL = FALSE, prefix = 'fX'),
                 colnames(sX, do.NULL = FALSE, prefix = 'sX') ) # generate the list of names for results
  if (!is.null(cX)) {res.names <- c(res.names,colnames(cX, do.NULL = FALSE, prefix = 'cX'))}
  names(betas$betas) <- res.names # change results' names
  names(betas$FSbetas)[2 : length(betas$FSbetas)] <- c(res.names, rep(" ", (length(betas$FSbetas) - length(res.names) - 1))) # change 1st stage results' names
  names(boot.errors) <- res.names # change std.errors' names
  elapsedTime = Sys.time() - Start # total running time
  out <- new("prod",
             Model = list(method = 'LP', FSbetas = betas$FSbetas, boot.repetitions = R, elapsed.time = elapsedTime,
                          theta0 = theta0 , opt = opt, opt.outcome = betas$opt.outcome, nCores = nCores),
             Data = list(Y = Y, free = fX, state = sX, proxy = pX, control = cX, idvar = idvar.original, timevar = timevar,
                         FSresiduals = betas$FSresiduals),
             Estimates = list(pars = betas$betas, std.errors = boot.errors))
  return(out)
}
# end of prodestLP #

# function to estimate and to bootstrap OP / LP #
finalOPLP <- function(ind, data, fnum, snum, cnum, opt, theta0, boot, tol, att){
  sampled <- .prepare_panel_resample(ind, data)
  data <- sampled$data
  newid <- sampled$idvar
  first.stage <- lm(data[,'Y'] ~ data[,grepl('regvars', colnames(data))], na.action = na.exclude)
  fX <- data[,grepl('free', colnames(data)), drop = FALSE]
  phi <- fitted(first.stage) # generate the fitted values of the first stage
  beta.free <-  coef(first.stage)[2:(1 + fnum)]
  if (cnum != 0) { # save betas of cX if included
    beta.control <- coef(first.stage)[(2 + fnum):(1 + fnum + cnum)]
  } else {
    beta.control <- NULL
  }
  if (is.null(theta0)) { # if not specified, starting points are the first stage + normal noise
    theta0 <- coef(first.stage)[(2 + fnum):(1 + fnum + snum)]  + rnorm((snum), 0, 0.01)
  }
  phi <- phi - (fX %*% beta.free) # "clean" the phi from the effects of free vars
  newtime <- data[,'timevar', drop = FALSE]
  rownames(phi) <- NULL
  rownames(newtime) <- NULL
  lag.phi <- lagPanel(value = phi, idvar = newid, timevar = newtime) #  # lag fitted values
  res <- data[,'Y', drop = FALSE] - (fX %*% beta.free) # "clean" the outcome from the effects of free vars
  state = data[, grepl('state', colnames(data)), drop = FALSE]
  lag.sX = data[, grepl('lag.sX', colnames(data)), drop = FALSE]
  Pr.hat <- data[, grepl('Pr.hat', colnames(data)), drop = FALSE]
  tmp.data <- model.frame(state ~ lag.sX + phi + lag.phi + res + Pr.hat)
  if (opt == 'optim'){
    try.state <- try(optim(theta0, gOPLP, method = "BFGS", mX = tmp.data$state, mlX = tmp.data$lag.sX, vphi = tmp.data$phi,
                            vlag.phi = tmp.data$lag.phi, vres = tmp.data$res, stol = tol, Pr.hat = tmp.data$Pr.hat, att = att), silent = TRUE)
    if (!inherits(try.state, "try-error")) {
      beta.state <- try.state$par
      opt.outcome <- try.state
    } else {
      beta.state <- matrix(NA,(snum),1)
      opt.outcome <- list(convergence = 99)
    } # error handling: if the optimization fails return missing values
  } else if (opt == 'DEoptim') {
    try.state <- try(DEoptim(fn = gOPLP, lower = c(theta0), upper = rep.int(1,length(theta0)),
                             mX = tmp.data$state, mlX = tmp.data$lag.sX,
                             vphi = tmp.data$phi, vlag.phi = tmp.data$lag.phi,
                             vres =  tmp.data$res, stol = tol, Pr.hat = tmp.data$Pr.hat, att = att, control = DEoptim.control(trace = FALSE)),
                     silent = TRUE)
    if (!inherits(try.state, "try-error")) {
      beta.state <- try.state$optim$bestmem
      opt.outcome <- try.state
    } else {
      beta.state <- matrix(NA,(snum), 1)
      opt.outcome <- list(convergence = 99)
    } # error handling: if the optimization fails return missing values
  } else if (opt == 'solnp') {
    try.state <- try(suppressWarnings(solnp(theta0, gOPLP, mX = tmp.data$state, mlX = tmp.data$lag.sX,
                           vphi = tmp.data$phi, vlag.phi = tmp.data$lag.phi,
                           vres =  tmp.data$res, stol = tol, Pr.hat = tmp.data$Pr.hat, att = att, control = list(trace = FALSE))), silent = TRUE)
    if (!inherits(try.state, "try-error")) {
      beta.state <- try.state$pars
      opt.outcome <- try.state
    } else {
      beta.state <- matrix(NA,(snum),1)
      opt.outcome <- list(convergence = 99)
    } # error handling: if the optimization fails return missing values
  }
  if (boot == FALSE){ # for the baseline estimation we want info on optimization, too
    return(list(betas = c(beta.free, beta.state, beta.control), opt.outcome = opt.outcome, FSbetas = coef(first.stage),
                FSresiduals = resid(first.stage)))
  } else{
    return(c(beta.free, beta.state, beta.control))
  }
}
# end of OP / LP final function #

# function to run the GMM estimation for OP and LP #
gOPLP <- function(vtheta, mX, mlX, vphi, vlag.phi, vres, stol, Pr.hat, att){
  Omega <- vphi - mX %*% vtheta
  Omega_lag <- vlag.phi - mlX %*% vtheta
  if (att == FALSE){
    Omega_lag_pol <- cbind(1, Omega_lag, Omega_lag^2, Omega_lag^3)
  } else {
    Omega_lag_pol <- cbind(1, Omega_lag, Omega_lag^2, Omega_lag^3, Pr.hat, Pr.hat^2, Pr.hat^3, Pr.hat * Omega_lag,
                           Pr.hat^2 * Omega_lag, Pr.hat * Omega_lag^2)
  }
  g_b <- solve(crossprod(Omega_lag_pol), tol = stol) %*% t(Omega_lag_pol) %*% Omega
  XI <- vres - (mX %*% vtheta) - (Omega_lag_pol %*% g_b)
  crit <- crossprod(XI)
  return(crit)
}
# end of GMM OPLP #

## Simulate a panel dataset  ##
panelSim <- function(N = 1000, T = 100, alphaL = .6, alphaK = .4, DGP = 1, rho = .7, sigeps = .1,
                     sigomg = .3, rholnw = .3){
  if (DGP != 1 & DGP != 2 & DGP != 3) stop('DGP must be 1, 2 or 3')
  if (T < 15) stop('T must be > 15 to yield a usable number of observations - i.e., 2 per panel.')
  siglnw.tot <- c(.1,0,.1) # parameters describing the different DGP
  timeb.tot <- c(.5,0,.5)
  sigoptl.tot <- c(0,.37,.37)
  measerrvec <- c(0,sqrt(.1),sqrt(.2),sqrt(.5)) # measurement error of the intermediate input
  starttime <- round(T*.9)
  numT <- T - starttime #this is the number of times to be used
  alpha0 <- 0 # starting points
  epsilon <- matrix(rnorm(N*T,0,sigeps),nrow=N,ncol=T) # matrix of random normal shocks
  siglnw <- siglnw.tot[DGP]
  timeb <- timeb.tot[DGP]
  sigoptl <- sigoptl.tot[DGP]
  # The next five lines define parameters necessary to subdivide the omega process into omega(t-1) -> omega(t-b) -> omega(t).  Note that this depends on the timeb (b) parameter, defined at top of program
  rhofirst <- rho^(1-timeb)
  rhosecond <- rho^timeb
  sigxi <- sqrt((1-rho^2)*sigomg^2)  # SD of innovation in omega - set such that variance of omega(t) is constant across time
  sigxifirst <- sqrt((1-rhofirst^2)*sigomg^2)
  sigxisecond <- sqrt((1-rhosecond^2)*sigomg^2)
  sigxilnw <- sqrt((1-rholnw^2)*siglnw^2) # SD of innovation in ln(wage) - set such that variance of ln(wage) is constant across time
  omgdata0 <- matrix(rnorm(N,0,sigomg),nrow=N,ncol=1)
  lnwdata0 <- matrix(rnorm(N,0,siglnw),nrow=N,ncol=1)
  # Period 1-b value of omega and Period 1 values of omega and ln(wage)
  omgdataminusb <- matrix(NA,N,T)
  omgdata <- matrix(NA,N,T)
  lnwdata <- matrix(NA,N,T)
  lnkdata <- matrix(NA,N,T)
  lnldata <- matrix(NA,N,T)
  lnpdata <- matrix(0,N,T)
  epsdata <- matrix(rnorm(N*T,0,sigeps),N,T)
  omgdataminusb[,1] = rhofirst*omgdata0 + matrix(rnorm(N,0,sigxifirst),nrow=N,ncol=1)
  omgdata[,1] = rhosecond*omgdataminusb[,1] + matrix(rnorm(N,0,sigxisecond),nrow=N,ncol=1)
  lnwdata[,1] = rholnw*lnwdata0 + matrix(rnorm(N,0,sigxilnw),nrow=N,ncol=1)
  # Simulate values of omega and ln(wage) for rest of time periods
  for (i in 1:N) {
    for (t in 2:T) {
      omgdataminusb[i,t] = rhofirst*omgdata[i,t-1] + rnorm(1,0,sigxifirst)
      omgdata[i,t] = rhosecond*omgdataminusb[i,t] + rnorm(1,0,sigxisecond)
      lnwdata[i,t] = rholnw*lnwdata[i,t-1] + rnorm(1,0,sigxilnw)
    }
  }
  lnkdata[,1] = matrix(-100,N,1)
  disc = 0.95 # Discount rate for dynamic programming problem
  delta = 0.2 # Depreciation rate for capital
  sigb = 0.6 # measures variation in capital adjustment cost across firms
  oneoverbiadj = exp(rnorm(N,0,sigb))
  # Various components of long expression for optimal investment choice (end of Appendix Section 7.3)
  squarebracketterm = (alphaL^(alphaL/(1-alphaL)))*exp(0.5*alphaL^2*sigoptl^2) - (alphaL^(1/(1-alphaL)))*exp(0.5*sigoptl^2)
  const1 = disc * (alphaK/(1-alphaL)) * (exp(alpha0)^(1/(1-alphaL))) * squarebracketterm
  horizon <- 100L
  steps <- seq_len(horizon)
  vec1 = (disc*(1-delta))^(steps - 1L)
  vec2 = cumsum(rholnw^(2*steps)) # Cumulative sum done through an upper triangular matrix --> check
  vec3 = sigxi^2 * c(0, cumsum(rho^(2*(seq_len(horizon - 1L) - 1L))))
  expterm3 = exp(0.5 * ((-alphaL)/(1-alphaL))^2 * sigxilnw^2 * vec2)
  expterm4 = exp(0.5 * (1/(1-alphaL))^2 * rhosecond^2 * ((sigxifirst^2)*rho^(2*steps) + vec3))
  expterm5 = exp((1/(1-alphaL))*(1/2)*sigxisecond^2)
  investmat = matrix(NA,N,T)
  for (i in 1:N) {
    for (t in 1:T) {
      expterm1 = exp( (1/(1-alphaL))*omgdata[i,t]*(rho^steps) ) # first term in exponent in second line
      expterm2 = exp( ((-alphaL)/(1-alphaL))*lnwdata[i,t]*(rholnw^steps) ) # second term in exponent in second line
      investmat[i,t] = oneoverbiadj[i]*const1*expterm5*sum(vec1*expterm1*expterm2*expterm3*expterm4)  # optimal investment
      if (t >= 2) {
        lnkdata[i,t] = log( (1-delta)*exp(lnkdata[i,t-1]) + (1-0*runif(1))*investmat[i,t-1] )
      }
    }
  }
  # Now generate levels of labor input for all firms and time periods - note: this choice depends on omega(t-b) since labor chosen at t-b
  for (i in 1:N) {
    for (t in 1:T) {
      lnldata[i,t] = ((sigxisecond^2)/2 + log(alphaL) + alpha0 + rhosecond*omgdataminusb[i,t] - lnwdata[i,t] + lnpdata[i,t] + (alphaK)*lnkdata[i,t])/(1-alphaL)
    }
  }
  # Now add potential optimization error to ln(labor) - note: sigoptl defined at top of program
  truelnldata = lnldata
  lnldata = lnldata + matrix(rnorm(N*T,0,sigoptl),N,T)
  # Generate levels of Output and Materials
  lnydata = alphaL*lnldata + alphaK*lnkdata + omgdata + epsdata
  lnmdata = alphaL*truelnldata + alphaK*lnkdata + omgdata
  truelnmdata = lnmdata
  withinvarlnm = withinvar(lnmdata[,(starttime+1):T]);
  # Note: Material input based on labor _without_ optimization error - this is necessary for LP to produce consistent estimates under DGP2 (without measurement error in materials) - see discussion in text. One interpretation of this is that econometrician observed planned (or ordered) materials (rather than materials actually used) */
  # Now add potential measurement error in material input - note that as described in the paper and table the amount is proportional to the within variance of ln(materials) */
  lnmdata2 = lnmdata + measerrvec[2]*matrix(rnorm(N*T,0,sqrt(withinvarlnm)),N,T)
  lnmdata3 = lnmdata + measerrvec[3]*matrix(rnorm(N*T,0,sqrt(withinvarlnm)),N,T)
  lnmdata4 = lnmdata + measerrvec[4]*matrix(rnorm(N*T,0,sqrt(withinvarlnm)),N,T)
  info <- list(lnydata,lnkdata,lnldata,lnmdata,lnmdata2,lnmdata3, lnmdata4)
  data <- matrix(unlist(lapply(info, function(x) { return(c(t((x[,(starttime+1):T])))) } )), ncol = 7)
  data <- data.frame(idvar = rep(1:N, each = numT), timevar = rep(1:numT,N), Y = data[,1], sX = data[,2], fX = data[,3],
                     pX1 = data[,4], pX2 = data[,5], pX3 = data[,6], pX4 = data[,7])
  return(data)
}

############ ACKERBERG-CAVES-FRAZER ###############

# Prepare the numeric matrices used by the ACF estimator.  Keeping this step
# separate makes the dimensions of control-variable models directly testable.
.prepare_acf_data <- function(Y, fX, sX, pX, idvar, timevar, cX = NULL){
  snum <- ncol(sX)
  fnum <- ncol(fX)
  cnum <- if (is.null(cX)) 0L else ncol(cX)

  polyframe <- data.frame(fX, sX, pX)
  mod <- model.matrix(~ .^2 - 1, data = polyframe)
  mod <- mod[match(rownames(polyframe), rownames(mod)), , drop = FALSE]

  # Put first-degree production inputs first.  This gives finalACF() stable
  # starting-value positions; lm() drops later duplicate first-degree columns.
  if (is.null(cX)) {
    regvars <- cbind(fX, sX, pX, mod, fX^2, sX^2, pX^2)
  } else {
    regvars <- cbind(fX, sX, cX, pX, mod, fX^2, sX^2, pX^2)
  }

  lag.sX <- sX
  for (i in seq_len(snum)) {
    lag.sX[, i] <- lagPanel(sX[, i], idvar = idvar, timevar = timevar)
  }

  lag.fX <- fX
  for (i in seq_len(fnum)) {
    lag.fX[, i] <- lagPanel(fX[, i], idvar = idvar, timevar = timevar)
  }

  idvar.internal <- .panel_id_code(idvar)
  if (is.null(cX)) {
    data <- as.matrix(data.frame(
      Y = Y,
      idvar = idvar.internal,
      timevar = timevar,
      Z = data.frame(lag.fX, sX),
      Xt = data.frame(fX, sX),
      lX = data.frame(lag.fX, lag.sX),
      regvars = regvars
    ))
  } else {
    lag.cX <- cX
    for (i in seq_len(cnum)) {
      lag.cX[, i] <- lagPanel(cX[, i], idvar = idvar, timevar = timevar)
    }

    # Controls are estimated production-function regressors.  Their current
    # and lagged values must therefore enter the second-stage matrices, while
    # their lags also enter the instrument matrix.
    data <- as.matrix(data.frame(
      Y = Y,
      idvar = idvar.internal,
      timevar = timevar,
      Z = data.frame(lag.fX, sX, lag.cX),
      Xt = data.frame(fX, sX, cX),
      lX = data.frame(lag.fX, lag.sX, lag.cX),
      regvars = regvars
    ))
  }

  storage.mode(data) <- "double"
  data
}

# function to estimate ACF model #
prodestACF <- function(Y, fX, sX, pX, idvar, timevar, R = 20, cX = NULL,
                       opt = 'optim', theta0 = NULL, cluster = NULL){
  Start <- Sys.time()
  Y <- checkM(Y)
  fX <- checkM(fX)
  sX <- checkM(sX)
  pX <- checkM(pX)
  idvar <- checkM(idvar)
  idvar.original <- idvar
  timevar <- checkM(timevar)
  snum <- ncol(sX)
  fnum <- ncol(fX)
  if (!is.null(cX)) {
    cX <- checkM(cX)
    cnum <- ncol(cX)
  } else {
    cnum <- 0L
  }
  n.parameters <- cnum + fnum + snum
  if (!is.null(theta0) && length(theta0) != n.parameters) {
    stop(paste0('theta0 length (', length(theta0),
                ') is inconsistent with the number of parameters (',
                n.parameters, ')'))
  }

  data <- .prepare_acf_data(
    Y = Y, fX = fX, sX = sX, pX = pX, idvar = idvar,
    timevar = timevar, cX = cX
  )

  betas <- finalACF(
    ind = TRUE, data = data, fnum = fnum, snum = snum, cnum = cnum,
    opt = opt, theta0 = theta0
  )
  boot.indices <- block.boot.resample(idvar, R)
  if (is.null(cluster)) {
    nCores <- NULL
    boot.betas <- matrix(
      unlist(lapply(
        boot.indices, finalACF, data = data, fnum = fnum, snum = snum,
        cnum = cnum, opt = opt, theta0 = theta0, boot = TRUE
      )),
      ncol = n.parameters, byrow = TRUE
    )
  } else {
    nCores <- length(cluster)
    clusterEvalQ(cl = cluster, library(prodest))
    boot.betas <- matrix(
      unlist(parLapply(
        cl = cluster, boot.indices, finalACF, data = data, fnum = fnum,
        snum = snum, cnum = cnum, opt = opt, theta0 = theta0,
        boot = TRUE
      )),
      ncol = n.parameters, byrow = TRUE
    )
  }
  boot.errors <- apply(boot.betas, 2, sd, na.rm = TRUE)
  res.names <- c(
    colnames(fX, do.NULL = FALSE, prefix = 'fX'),
    colnames(sX, do.NULL = FALSE, prefix = 'sX')
  )
  if (!is.null(cX)) {
    res.names <- c(res.names, colnames(cX, do.NULL = FALSE, prefix = 'cX'))
  }
  names(betas$betas) <- res.names
  names(boot.errors) <- res.names
  elapsedTime <- Sys.time() - Start
  out <- new(
    "prod",
    Model = list(
      method = 'ACF', FSbetas = NA, boot.repetitions = R,
      elapsed.time = elapsedTime, theta0 = theta0, opt = opt,
      opt.outcome = betas$opt.outcome, nCores = nCores
    ),
    Data = list(
      Y = Y, free = fX, state = sX, proxy = pX, control = cX,
      idvar = idvar.original, timevar = timevar,
      FSresiduals = betas$FSresiduals
    ),
    Estimates = list(pars = betas$betas, std.errors = boot.errors)
  )
  return(out)
}
# end of prodestACF #

# function to estimate and bootstrap ACF #
finalACF <- function(ind, data, fnum, snum, cnum, opt, theta0,
                     boot = FALSE){
  sampled <- .prepare_panel_resample(ind, data)
  data <- sampled$data
  newid <- sampled$idvar
  n.parameters <- fnum + snum + cnum

  first.stage <- lm(
    data[, 'Y', drop = FALSE] ~
      data[, grepl('regvars', colnames(data)), drop = FALSE],
    na.action = na.exclude
  )
  phi <- fitted(first.stage)
  if (is.null(theta0)) {
    theta0 <- as.numeric(coef(first.stage)[2:(1 + n.parameters)]) +
      rnorm(n.parameters, 0, 0.01)
  }
  newtime <- data[, 'timevar', drop = FALSE]
  rownames(phi) <- NULL
  rownames(newtime) <- NULL
  lag.phi <- lagPanel(idvar = newid, timevar = newtime, value = phi)
  Z <- data[, grepl('Z', colnames(data)), drop = FALSE]
  X <- data[, grepl('Xt', colnames(data)), drop = FALSE]
  lX <- data[, grepl('lX', colnames(data)), drop = FALSE]
  tmp.data <- model.frame(Z ~ X + lX + phi + lag.phi)
  W <- solve(crossprod(tmp.data$Z)) / nrow(tmp.data$Z)

  if (opt == 'optim') {
    try.out <- try(
      optim(
        theta0, gACF, method = "BFGS", mZ = tmp.data$Z, mW = W,
        mX = tmp.data$X, mlX = tmp.data$lX, vphi = tmp.data$phi,
        vlag.phi = tmp.data$lag.phi
      ),
      silent = TRUE
    )
    if (!inherits(try.out, "try-error")) {
      betas <- try.out$par
      opt.outcome <- try.out
    } else {
      betas <- rep(NA_real_, n.parameters)
      opt.outcome <- list(convergence = 999)
    }
  } else if (opt == 'DEoptim') {
    try.out <- try(
      DEoptim(
        gACF, lower = theta0, upper = rep.int(1, length(theta0)),
        mZ = tmp.data$Z, mW = W, mX = tmp.data$X, mlX = tmp.data$lX,
        vphi = tmp.data$phi, vlag.phi = tmp.data$lag.phi,
        control = DEoptim.control(trace = FALSE)
      ),
      silent = TRUE
    )
    if (!inherits(try.out, "try-error")) {
      betas <- try.out$optim$bestmem
      opt.outcome <- try.out
    } else {
      betas <- rep(NA_real_, n.parameters)
      opt.outcome <- list(convergence = 99)
    }
  } else if (opt == 'solnp') {
    try.out <- try(
      suppressWarnings(solnp(
        theta0, gACF, mZ = tmp.data$Z, mW = W, mX = tmp.data$X,
        mlX = tmp.data$lX, vphi = tmp.data$phi,
        vlag.phi = tmp.data$lag.phi, control = list(trace = FALSE)
      )),
      silent = TRUE
    )
    if (!inherits(try.out, "try-error")) {
      betas <- try.out$pars
      opt.outcome <- try.out
    } else {
      betas <- rep(NA_real_, n.parameters)
      opt.outcome <- list(convergence = 999)
    }
  } else {
    stop("opt must be one of 'optim', 'DEoptim' or 'solnp'")
  }

  if (!boot) {
    return(list(
      betas = betas, opt.outcome = opt.outcome,
      FSresiduals = resid(first.stage)
    ))
  }
  betas
}
# end of ACF final function #

# function to run the GMM estimation for ACF #
gACF <- function(theta, mZ, mW, mX, mlX, vphi, vlag.phi){
  theta <- as.numeric(theta)
  if (ncol(mX) != length(theta) || ncol(mlX) != length(theta)) {
    stop("theta must contain one value for each column of mX and mlX")
  }
  Omega <- vphi - mX %*% theta
  Omega_lag <- vlag.phi - mlX %*% theta
  Omega_lag_pol <- cbind(1, Omega_lag, Omega_lag^2, Omega_lag^3)
  g_b <- solve(crossprod(Omega_lag_pol)) %*%
    t(Omega_lag_pol) %*% Omega
  XI <- Omega - Omega_lag_pol %*% g_b
  crit <- t(crossprod(mZ, XI)) %*% mW %*% crossprod(mZ, XI)
  return(crit)
}
# end of GMM ACF #

# function to check and change input to matrix #
checkM <- function(input){ # , inputname = NA
  if (!is.matrix(input)) {
    out <- as.matrix(input)
  } else{
    out <- input
  }
  colnames(out) <- NULL
  return(out)
}
# end check matrix fun #

# Convert arbitrary one-column panel identifiers to consecutive integer codes.
# Character and factor labels are accepted.  The codes are used only in
# internal numeric work matrices; fitted objects retain the original labels.
.panel_id_code <- function(idvar){
  id_matrix <- if (is.matrix(idvar)) idvar else as.matrix(idvar)
  if (ncol(id_matrix) != 1L) {
    stop("idvar must contain exactly one column")
  }

  ids <- as.vector(id_matrix)
  if (length(ids) == 0L) {
    stop("idvar must contain at least one observation")
  }
  if (any(is.na(ids))) {
    stop("idvar must not contain missing values")
  }

  out <- matrix(match(ids, unique(ids)), ncol = 1L)
  rownames(out) <- rownames(id_matrix)
  out
}

# Apply either an ordinary logical row selection or a block-bootstrap index.
.prepare_panel_resample <- function(ind, data){
  if (is.logical(ind)) {
    if (length(ind) == 1L) {
      ind <- rep(ind, nrow(data))
    }
    if (length(ind) != nrow(data) || any(is.na(ind))) {
      stop("logical ind must have length one or one value per data row")
    }
    rows <- which(ind)
    newid <- data[rows, "idvar", drop = FALSE]
  } else {
    rows <- as.integer(ind)
    if (length(rows) == 0L || any(is.na(rows)) ||
        any(rows < 1L | rows > nrow(data))) {
      stop("ind contains invalid row indices")
    }
    bootstrap_ids <- rownames(ind)
    if (is.null(bootstrap_ids) || length(bootstrap_ids) != length(rows)) {
      stop("bootstrap indices must carry resampled panel IDs as row names")
    }
    newid <- matrix(as.numeric(bootstrap_ids), ncol = 1L)
    if (any(is.na(newid))) {
      stop("bootstrap panel IDs must be numeric")
    }
  }

  list(data = data[rows, , drop = FALSE], idvar = newid)
}

# function to check and change input - dummy variables - to matrix #
checkMD <- function(input){ # , inputname = NA
  if (!is.matrix(input)) {
    out <- as.matrix(input)
  } else{
    out <- input
  }
  colnames(out) <- NULL
  if (any( (out != 0) & (out != 1) ) ){ # if exit is not binary, stop the routine
    nm <-deparse(substitute(input))
    stop(paste(nm, "is not binary"))
  }
  return(out)
}
# end check matrix fun #

# function to lag variables within a panel #
lagPanel <- function(idvar, timevar, value){
  idvar <- as.vector(idvar)
  timevar <- as.vector(timevar)

  if (length(idvar) != length(timevar)) {
    stop("idvar and timevar must have the same length")
  }
  if (!is.numeric(timevar)) {
    stop("timevar must be numeric")
  }

  vector_input <- is.null(dim(value))
  value_matrix <- if (vector_input) matrix(value, ncol = 1L) else as.matrix(value)

  if (nrow(value_matrix) != length(idvar)) {
    stop("value must have one row for each idvar/timevar observation")
  }

  valid <- !is.na(idvar) & !is.na(timevar)
  source_key <- rep(NA_character_, length(idvar))
  source_key[valid] <- paste(idvar[valid], timevar[valid], sep = "\r")

  if (anyDuplicated(source_key[valid])) {
    stop("idvar and timevar must uniquely identify observations")
  }

  target_key <- rep(NA_character_, length(idvar))
  target_key[valid] <- paste(idvar[valid], timevar[valid] - 1, sep = "\r")
  source_row <- match(target_key, source_key)

  out <- value_matrix[source_row, , drop = FALSE]
  out[!valid, ] <- NA

  if (vector_input) {
    out <- as.vector(out[, 1L])
    names(out) <- names(value)
    return(out)
  }

  input_names <- colnames(value_matrix)
  if (is.null(input_names)) {
    input_names <- paste0("V", seq_len(ncol(value_matrix)))
  }
  colnames(out) <- paste0("l", input_names)
  rownames(out) <- rownames(value_matrix)
  out
}
# end of lag panel #

# function to demean a vector and take the variance #
withinvar <- function(inmat){
  devmat = inmat - mean(inmat) # demean the vector
  return(var(c(devmat)))
}
# end of within variance function #

# boot resampling on IDs: bootstrapping on individuals #
block.boot.resample <- function(idvar, R){
  id_matrix <- if (is.matrix(idvar)) idvar else as.matrix(idvar)
  if (ncol(id_matrix) != 1L) {
    stop("idvar must contain exactly one column")
  }

  ids <- as.vector(id_matrix)
  if (length(ids) == 0L) {
    stop("idvar must contain at least one observation")
  }
  if (any(is.na(ids))) {
    stop("idvar must not contain missing values")
  }
  if (length(R) != 1L || is.na(R) || !is.finite(R) ||
      R < 1L || R != as.integer(R)) {
    stop("R must be a positive integer")
  }
  R <- as.integer(R)

  unique_ids <- unique(ids)
  panel_indices <- lapply(unique_ids, function(x) which(ids == x))
  n_panels <- length(panel_indices)

  lapply(seq_len(R), function(r) {
    sampled_panels <- sample.int(n_panels, size = n_panels, replace = TRUE)
    sampled_indices <- unlist(panel_indices[sampled_panels], use.names = FALSE)
    bootstrap_ids <- unlist(
      Map(function(new_id, sampled_panel) {
        rep.int(new_id, length(panel_indices[[sampled_panel]]))
      }, seq_along(sampled_panels), sampled_panels),
      use.names = FALSE
    )

    matrix(
      sampled_indices,
      ncol = 1L,
      dimnames = list(as.character(bootstrap_ids), NULL)
    )
  })
}
# end of block bootstrap function #

# function to compute the weighting matrix #
weightM <- function(Y, X1, X2, Z1, Z2, betas, numR, SE = FALSE){
  k1 <- ncol(X1)
  N <- nrow(X1)
  R1t <- Y - X1 %*% betas[1 : k1, drop = FALSE]
  R2t <- Y - X2 %*% c(betas[1 : numR], betas[(k1 + 1) : length(betas), drop = FALSE]) # (fnum + snum + cnum + 1)
  u <- c(R1t, R2t) # alternative, still working
  Z <- as.matrix( bdiag(Z1, Z2)) # drop the collinear constant
  sigma.rs <- (t(u) %*% u)
  S <- sigma.rs[1] * ( ( t(Z) %*% Z) ) # /N
  if (SE == TRUE){
    dX <- rbind( cbind( X1, matrix(0, N, (ncol(X2) - numR) ) ),
                 cbind( X2[, 1 : numR], matrix(0 , N, (ncol(X1) - numR) ), X2[,(numR + 1) : ncol(X2)]) ) # generate a "quasi-block" matrix with common columns NON-BLOCK
    var.beta <- (1/N) * solve( ( t(dX) %*% Z ) %*% solve(S) %*% (t(Z) %*% dX) ) # compute varCovar matrix
    st.errors <- sqrt(diag(var.beta))
    return(st.errors)
  } else {
    W = solve(S)
    return(W)
  }
}
# end of weighting matrix function #

# function to print lateX table of results #
printProd <- function(mods, modnames = NULL, parnames = NULL, outfile = NULL, ptime = FALSE, nboot = FALSE, screen = FALSE){
  if (!is.null(outfile)) (sink(outfile)) # write on a text file
  numMods <- length(mods)
  numPars <- length(mods[[1]]@Estimates$pars)
  if (screen == FALSE){
    cat(paste('\\begin{tabular}{', paste(rep('c',(numMods*2+1)), collapse = ''),'}',
              '\\hline\\hline', sep = '')) # print tabular header
    nm <- '\n'
    obs <- '\nN'
    time <- '\nTime'
    boot <- '\nBootRep'
    for (m in 1:numMods){ # generate first and last row: names (methods or user-supplied) and observations
      if (is.null(modnames)){
        nm <- paste(nm, mods[[m]]@Model$method, sep = ' & & ')
      }else{
        nm <- paste(nm, modnames[m], sep = ' & & ')
      }
      obs <- paste(obs, length(mods[[m]]@Data$Y), sep = ' & & ')
      time <- paste(time, round(mods[[m]]@Model$elapsed.time[[1]], digits = 2), sep = ' & & ')
      boot <- paste(boot, mods[[m]]@Model$boot.repetitions, sep = ' & & ')
    }
    nm <- paste(nm, '\\\\\\hline')
    obs <- paste(obs, '\\\\\\hline\\hline')
    cat(nm)
    for (p in 1:numPars){ # generate the table body row by row: names (vars or user-supplied),
      if (is.null(parnames)){
        betas <- paste('\n', names(mods[[1]]@Estimates$pars)[p])
      }
      else{
        betas <- paste('\n', parnames[p])
      }
      sigmas <- '\n'
      blank <- '\n'
      for (m in 1:numMods){
        betas <- paste(betas, round(mods[[m]]@Estimates$pars[p],digits = 3), sep = ' & & ')
        sigma <- paste('(', round(mods[[m]]@Estimates$std.errors[p],digits = 3), ')', sep = '')
        sigmas <- paste(sigmas, sigma , sep = ' & & ')
        blank <- paste(blank, ' & ',  sep = '')
      }
      betas <- paste(betas, '\\\\')
      sigmas <- paste(sigmas, '\\\\')
      blank <- paste(blank, '\\\\')
      cat(betas)
      cat(sigmas)
      cat(blank)
    }
    cat(blank)
    if (ptime == TRUE) (cat(paste(time, '\\\\', sep = '')))
    if (nboot == TRUE) (cat(paste(boot, '\\\\', sep = '')))
    cat(obs)
    cat('\n\\end{tabular}')
    if (!is.null(outfile)) (sink())
  } else{
    cat(paste(rep('--', (numMods*3+1), sep = ''))) # print
    nm <- '\n'
    obs <- '\nN'
    time <- '\nTime'
    boot <- '\nBootRep'
    for (m in 1:numMods){ # generate first and last row: names (methods or user-supplied) and observations
      if (is.null(modnames)){
        nm <- paste(nm, mods[[m]]@Model$method, sep = '       ')
      }else{
        nm <- paste(nm, modnames[m], sep = '  ')
      }
      obs <- paste(obs, length(mods[[m]]@Data$Y), sep = '    ')
      time <- paste(time, round(mods[[m]]@Model$elapsed.time[[1]], digits = 2), sep = '  ')
      boot <- paste(boot, mods[[m]]@Model$boot.repetitions, sep = '  ')
    }
    cat(nm)
    cat('\n')
    cat(paste(rep('--',(numMods*3+1), sep = ''))) # print
    for (p in 1:numPars){ # generate the table body row by row: names (vars or user-supplied),
      if (is.null(parnames)){
        betas <- paste('\n', names(mods[[1]]@Estimates$pars)[p])
      }
      else{
        betas <- paste('\n', parnames[p])
      }
      sigmas <- '\n   '
      blank <- ''
      for (m in 1:numMods){
        betas <- paste(betas, round(mods[[m]]@Estimates$pars[p],digits = 3), sep = '   ')
        sigma <- paste('(', round(mods[[m]]@Estimates$std.errors[p],digits = 3), ')', sep = '')
        sigmas <- paste(sigmas, sigma , sep = ' ')
        blank <- paste(blank, rep('--',(numMods*2+1)),  sep = '')
      }
      cat(betas)
      cat(sigmas)
      cat('\n')
      cat(blank)
    }
    if (ptime == TRUE) (cat(paste(time, '', sep = '')))
    if (nboot == TRUE) (cat(paste(boot, '', sep = '')))
    cat(obs)
    cat('\n')
    cat(blank)
  }
}
# end of latex print table #

\name{prod}
\Rdversion{0.1}
\docType{class}
\alias{prod}
\alias{prod-class}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Class for Prodest Fitted object}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Class for prodest fitted objects.
}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
%%%%%%%%%%%%%%%% SLOTS %%%%%%%%%%%%%%%%%%%
\section{Slots}{
  \describe{
    %
    \item{\code{Model}:}{Object of class \code{list}. Contains information about the model and the optimization procedure:
        \itemize{
          \item \code{method}: \code{string} The method used in estimation.
          \item \code{FSbetas}: \code{numeric} First-stage estimated parameters.
          \item \code{boot.repetitions}: \code{numeric} Number of bootstrap repetitions.
          \item \code{elapsed.time}: \code{numeric} Time - in seconds - required for estimation.
          \item \code{theta0}: \code{numeric} Vector of Second-stage optimization starting points.
          \item \code{opt}: \code{string} Optimizer used for the Second-stage.
          \item \code{seed}: \code{numeric} seed set.
          \item \code{opt.outcome}: \code{list} Optimization outcome (depends on optimizer choice).
        }
    }
    %
    \item{\code{Data}:}{Object of class \code{list}. Contains:
      \itemize{
        \item \code{Y}: \code{numeric} Dependent variable - Value added.
        \item \code{free}: \code{matrix} Free variable(s).
        \item \code{state}: \code{matrix} State variable(s).
        \item \code{proxy}: \code{matrix} Proxy variable(s).
        \item \code{control}: \code{matrix} Control variable(s).
        \item \code{idvar}: \code{numeric} Panel identifiers.
        \item \code{timevar}: \code{numeric} Time identifiers.
        \item \code{FSresiduals}: \code{numeric} First-Stage residuals.
      }
    }
    %
    \item{\code{Estimates}:}{Object of class \code{list}. Contains:
      \itemize{
        \item \code{pars}: \code{numeric} Estimated parameters for the variables of interest.
        \item \code{std.errors}: \code{numeric} Estimated standard errors for the variables of interest.
      }
    }
  }
}
%%%%%%%%%%%%%%%% METHODS %%%%%%%%%%%%%%%%%%%
\section{Methods}{
  \itemize{
    \item \code{show} \code{signature(object = 'prod')}: Show table with the method, the estimated parameters and their standard errors.
    \item \code{summary} \code{signature(object = 'prod')}: Show table with method, parameters, std.errors and auxiliary information on model and optimization.
    \item \code{FSres} \code{signature(object = 'prod')}: Extract First-Stage residual vector.
    \item \code{omega} \code{signature(object = 'prod')}: Extract estimated productivity vector.
    \item \code{coef} \code{signature(object = 'prod')}: Extract estimated coefficients.
  }
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}
\keyword{classes}

\name{prodestWRDG_GMM}
\alias{prodestWRDG_GMM}
\title{
  Estimate productivity - Wooldridge method
}
\description{
  The \code{prodestWRDG_GMM()} function accepts at least 6 objects (id, time, output, free, state and proxy variables), and returns a \code{prod} object of class \code{S3} with three elements: (i) a list of model-related objects, (ii) a list with the data used in the estimation and estimated vectors of first-stage residuals, and (iii) a list with the estimated parameters and their bootstrapped standard errors.
}
\usage{
  prodestWRDG_GMM(Y, fX, sX, pX, idvar, timevar, cX = NULL, tol = 1e-100)
}
\arguments{
  \item{Y      }{
    the vector of value added log output.}
  %
  \item{fX     }{
    the vector/matrix/dataframe of log free variables.}
  %
  \item{sX     }{
    the vector/matrix/dataframe of log state variables.}
  %
  \item{pX    }{
    the vector/matrix/dataframe of log proxy variables.}
  %
  \item{cX     }{
    the vector/matrix/dataframe of control variables. By default \code{cX= NULL}.}
  %
  \item{idvar  }{
    the vector/matrix/dataframe identifying individual panels.}
  %
  \item{timevar}{
    the vector/matrix/dataframe identifying time.}
  %
  \item{tol}{
    optimizer tolerance. By default \code{tol = 1e-100}.}
  %
}
%%%%%%%%% DETAILS %%%%%%%%%%%
  \details{
    Consider a Cobb-Douglas production technology for firm \eqn{i} at time \eqn{t}
    \itemize{
      \item \eqn{y_{it} = \alpha + w_{it}\beta + k_{it}\gamma + \omega_{it} + \epsilon_{it}}
    }
    where \eqn{y_{it}} is the (log) output, \eqn{w_{it}} is a \eqn{1 \times J}{1 x J} vector of (log) free variables, \eqn{k_{it}} is a \eqn{1 \times K}{1 x K} vector of state variables, and \eqn{\epsilon_{it}} is a normally distributed idiosyncratic error term.
    The unobserved technical efficiency parameter \eqn{\omega_{it}} evolves according to a first-order Markov process:
      \itemize{
        \item \eqn{\omega_{it} = E(\omega_{it} | \omega_{it-1}) + u_{it} = g(\omega_{it-1}) + u_{it}}
      }
    and \eqn{u_{it}} is a random shock component assumed to be uncorrelated with the technical efficiency, the state variables in \eqn{k_{it}} and the lagged free variables \eqn{w_{it-1}}.
    Wooldridge method allows to jointly estimate OP/LP two stages jointly in a system of two equations. It relies on the following set of assumptions:
      \itemize{
        \item a) \eqn{\omega_{it} = g(x_{it} , p_{it})}: productivity is an unknown function \eqn{g(.)} of state and a proxy variables;
\item b) \eqn{E(\omega_{it} | \omega_{it-1)}=f[\omega_{it-1}]}, productivity is an unknown function \eqn{f[.]} of lagged productivity, \eqn{\omega_{it-1}}.
      }
    Under the above set of assumptions, It is possible to construct a system gmm using the  vector of residuals from
    \itemize{
      \item \eqn{r_{1it} = y_{it} - \alpha - w_{it}\beta - x_{it}\gamma - g(x_{it} , p_{it}) }
      \item \eqn{r_{2it} = y_{it} - \alpha - w_{it}\beta - x_{it}\gamma - f[g(x_{it-1} , p_{it-1})]}
    }
    where the unknown function \eqn{f(.)} is approximated by a n-th order polynomial and \eqn{g(x_{it} , m_{it}) = \lambda_0 + c(x_{it} , m_{it})\lambda}. In particular, \eqn{g(x_{it} , m_{it})} is a linear combination of functions in \eqn{(x_{it} , m_{it})}
    and \eqn{c_{it}} are the addends of this linear combination. The residuals \eqn{r_{it}} are used to set the moment conditions
    \itemize{
      \item \eqn{E(Z_{it}*r_{it}) =0}
    }
    with the following set of instruments:
      \itemize{
        \item \eqn{Z1_{it} = (1, w_{it}, x_{it}, c_{it})}
        \item \eqn{Z2_{it} = (w_{it-1}, c_{it}, c_{it})}
      }
  }
%%%%%%%%% VALUE %%%%%%%%%%%
  \value{
    The output of the function \code{prodestWRDG} is a member of the \code{S3} class \pkg{prod}. More precisely, is a list (of length 3) containing the following elements:

    \code{Model}, a list containing:
      \itemize{
        \item \code{method:} a string describing the method ('WRDG').
        \item \code{elapsed.time:} time elapsed during the estimation.
        \item \code{opt.outcome:} optimization outcome.
      }

    \code{Data}, a list containing:
      \itemize{
        \item \code{Y:} the vector of value added log output.
        \item \code{free:} the vector/matrix/dataframe of log free variables.
        \item \code{state:} the vector/matrix/dataframe of log state variables.
        \item \code{proxy:} the vector/matrix/dataframe of log proxy variables.
        \item \code{control:} the vector/matrix/dataframe of log control variables.
        \item \code{idvar:} the vector/matrix/dataframe identifying individual panels.
        \item \code{timevar:} the vector/matrix/dataframe identifying time.
      }

      \code{Estimates}, a list containing:
      \itemize{
        \item \code{pars:} the vector of estimated coefficients.
        \item \code{std.errors:} the vector of bootstrapped standard errors.
      }

     Members of class \code{prod} have an \code{omega} method returning a numeric object with the estimated productivity - that is: \eqn{\omega_{it} = y_{it} - (\alpha + w_{it}\beta + k_{it}\gamma)}.
    \code{FSres} method returns a numeric object with the residuals of the first stage regression, while \code{summary}, \code{show} and \code{coef} methods are implemented and work as usual.
  }
%%%%%%%%% AUTHOR %%%%%%%%%%%
  \author{
    Gabriele Rovigatti
  }
%%%%%%%%% REFERENCES %%%%%%%%%%%
  \references{
    Wooldridge, J M (2009).
    "On estimating firm-level production functions using proxy variables to control for unobservables."
    Economics Letters, 104, 112-114.
  }
%%%%%%%%% EXAMPLES %%%%%%%%%%%
  \examples{
    data("chilean")

    # we fit a model with two free (skilled and unskilled), one state (capital)
    # and one proxy variable (materials)

    WRDG.GMM.fit <- prodestWRDG_GMM(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2),
                              chilean$sX, chilean$pX, chilean$idvar, chilean$timevar)

    # show results
    WRDG.GMM.fit

    \donttest{
      # estimate a panel dataset - DGP1, various measurement errors - and run the estimation
      sim <- panelSim()

      WRDG.GMM.sim1 <- prodestWRDG_GMM(sim$Y, sim$fX, sim$sX, sim$pX1, sim$idvar, sim$timevar)
      WRDG.GMM.sim2 <- prodestWRDG_GMM(sim$Y, sim$fX, sim$sX, sim$pX2, sim$idvar, sim$timevar)
      WRDG.GMM.sim3 <- prodestWRDG_GMM(sim$Y, sim$fX, sim$sX, sim$pX3, sim$idvar, sim$timevar)
      WRDG.GMM.sim4 <- prodestWRDG_GMM(sim$Y, sim$fX, sim$sX, sim$pX4, sim$idvar, sim$timevar)

      # show results in .tex tabular format
      printProd(list(WRDG.GMM.sim1, WRDG.GMM.sim2, WRDG.GMM.sim3, WRDG.GMM.sim4),
                      parnames = c('Free','State'))
    }
 }

\name{summary}
\alias{summary,prod-method}
\alias{summary}
\docType{methods}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Print a table with a summary of results}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  This method allows the user to print a table with a summary of the results within an \code{S4} \link{prod} object: the parameter estimates, the method, the number of observations, the time elapsed, the number of bootstrap repetitions, the first stage estimates and the starting points.
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{
  \S4method{summary}{prod}(object, ...)
}
%%%%%%%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%%%%
\arguments{
  %
  \item{object}{
    object of class \link{prod}.
  }
  %
  \item{...}{
    Additional arguments.
  }
}
%%%%%%%%%%%%%%%% DETAILS %%%%%%%%%%%%%%%%%%%
\details{
  \code{summary} accepts an \code{S4} \link{prod} object and prints a table with the results.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{checkM}
\alias{checkM}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Change input to matrix}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Function to transform all input to matrix.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  checkM(input)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  %
  \item{input}{
    An \code{R} object. Can be a matrix/dataframe/vector/scalar.
  }
  %
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  \code{checkM()} accepts one input and - if \code{input} is a matrix - returns it without column names, otherwise transforms it into a matrix and returns it without column names.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{chilean}
\docType{data}
\alias{chilean}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Data: Chilean firm-level production data, 1996--2006}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Sectoral subsample of Chilean firm-level production data covering 1996--2006.
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{data("chilean")}
%%%%%%%%%%%%%%%% FORMAT %%%%%%%%%%%%%%%%%%%
\format{
  A \link{data.frame} with 2,544 observations and 8 variables:
  \describe{
    \item{Y}{Log value added.}
    \item{sX}{Log capital.}
    \item{fX1}{Log skilled labour.}
    \item{fX2}{Log unskilled labour.}
    \item{pX}{Log materials.}
    \item{inv}{Log investment.}
    \item{idvar}{Firm identifier.}
    \item{timevar}{Year.}
  }
}
%%%%%%%%% REFERENCES %%%%%%%%%
\references{
  Instituto Nacional de Estadisticas de Chile, Encuesta Nacional Industrial Anual (ENIA).
}
%%%%%%%%% KEYWORDS %%%%%%%%%
\keyword{datasets}

\name{gOPLP}
\alias{gOPLP}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{OP and LP Second Stage - GMM estimation}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  \code{gOPLP} returns the second stage parameters estimates of both OP and LP models. It is part of both \code{prodestOP()} and \code{prodestsLP()} routines.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  gOPLP(vtheta, mX, mlX, vphi, vlag.phi, vres, stol, Pr.hat, att)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  %
  \item{vtheta}{
    Vector of parameters to be estimated.
  }
  %
  \item{mX}{
    Matrix of regressors.
  }
  %
  \item{mlX}{
    matrix of lagged regressors.
  }
  %
  \item{vphi}{
    Vector of fitted polynomial.
  }
  %
  \item{vlag.phi}{
    Lagged vector of fitted polynomial.
  }
  %
  \item{vres}{
    Vector of residuals of the free variables.
  }
  %
  \item{stol}{
    Number setting the tolerance of the routine.
  }
  %
  \item{Pr.hat}{
    Vector of fitted exit probabilities.
  }
  %
  \item{att}{
    Indicator for attrition in the data - i.e., if  firms exit the market.
  }
  %
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  \code{gOPLP()} estimates the second stage of OP and LP routines. It accepts 7 inputs, generates and optimizes over the group of moment functions \eqn{E(e_{it} X^k_{it})}{E(e_it X_it^k)}.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{withinvar}
\alias{withinvar}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Generate the variance of the demeaned variable}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  \code{withinvar()} subtracts the mean of a vector from the vector itself, and then returns its variance.
}
%%%%%%%%% USAGE %%%%%%%%%%%
  \usage{
    withinvar(inmat)
  }
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
  \arguments{
    %
    \item{inmat}{
      Vector of log(value added output).
    }
    %
  }
%%%%%%%%% DETAILS %%%%%%%%%
  \details{
    \code{withinvar()} accepts a vector as input, then subtracts the mean from it and returns the variance.
  }
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{prodestROB}
\alias{prodestROB}
\title{
  Estimate productivity - Robinson-Wooldridge method
}
\description{
  The \code{prodestROB()} function accepts at least 6 objects (id, time, output, free, state and proxy variables), and returns a \code{prod} object of class \code{S3} with three elements: (i) a list of model-related objects, (ii) a list with the data used in the estimation and estimated vectors of first-stage residuals, and (iii) a list with the estimated parameters and their bootstrapped standard errors.
}
\usage{
  prodestROB(Y, fX, sX, pX, idvar, timevar, cX = NULL)
}
\arguments{
  \item{Y      }{
    the vector of value added log output.}
  %
  \item{fX     }{
    the vector/matrix/dataframe of log free variables.}
  %
  \item{sX     }{
    the vector/matrix/dataframe of log state variables.}
  %
  \item{pX    }{
    the vector/matrix/dataframe of log proxy variables.}
  %
  \item{cX     }{
    the vector/matrix/dataframe of control variables. By default \code{cX= NULL}.}
  %
  \item{idvar  }{
    the vector/matrix/dataframe identifying individual panels.}
  %
  \item{timevar}{
    the vector/matrix/dataframe identifying time.}
  %
}
%%%%%%%%% DETAILS %%%%%%%%%%%
  \details{
    Consider a Cobb-Douglas production technology for firm \eqn{i} at time \eqn{t}
    \itemize{
      \item \eqn{y_{it} = \alpha + w_{it}\beta + k_{it}\gamma + \omega_{it} + \epsilon_{it}}
    }
    where \eqn{y_{it}} is the (log) output, \eqn{w_{it}} is a \eqn{1 \times J}{1 x J} vector of (log) free variables, \eqn{k_{it}} is a \eqn{1 \times K}{1 x K} vector of state variables, and \eqn{\epsilon_{it}} is a normally distributed idiosyncratic error term.
    The unobserved technical efficiency parameter \eqn{\omega_{it}} evolves according to a first-order Markov process:
      \itemize{
        \item \eqn{\omega_{it} = E(\omega_{it} | \omega_{it-1}) + u_{it} = g(\omega_{it-1}) + u_{it}}
      }
    and \eqn{u_{it}} is a random shock component assumed to be uncorrelated with the technical efficiency, the state variables in \eqn{k_{it}} and the lagged free variables \eqn{w_{it-1}}.
    Wooldridge method allows to jointly estimate OP/LP two stages jointly in a system of two equations. It relies on the following set of assumptions:
      \itemize{
        \item a) \eqn{\omega_{it} = g(x_{it} , p_{it})}: productivity is an unknown function \eqn{g(.)} of state and a proxy variables;
\item b) \eqn{E(\omega_{it} | \omega_{it-1)}=f[\omega_{it-1}]}, productivity is an unknown function \eqn{f[.]} of lagged productivity, \eqn{\omega_{it-1}}.
      }
    Under the above set of assumptions, It is possible to construct a system gmm using the  vector of residuals from
    \itemize{
      \item \eqn{r_{1it} = y_{it} - \alpha - w_{it}\beta - x_{it}\gamma - g(x_{it} , p_{it}) }
      \item \eqn{r_{2it} = y_{it} - \alpha - w_{it}\beta - x_{it}\gamma - f[g(x_{it-1} , p_{it-1})]}
    }
    where the unknown function \eqn{f(.)} is approximated by a n-th order polynomial and \eqn{g(x_{it} , m_{it}) = \lambda_{0} + c(x_{it} , m_{it})\lambda}. In particular, \eqn{g(x_{it} , m_{it})} is a linear combination of functions in \eqn{(x_{it} , m_{it})}
    and \eqn{c_{it}} are the addends of this linear combination. The residuals \eqn{r_{it}} are used to set the moment conditions
    \itemize{
      \item \eqn{E(Z_{it}*r_{it}) =0}
    }
    with the following set of instruments:
      \itemize{
        \item \eqn{Z1_{it} = (1, w_{it}, x_{it}, c_{it})}
        \item \eqn{Z2_{it} = (w_{it-1}, c_{it}, c_{it})}
      }
    According to the input timing in ACF, the first equation proposed by Wooldridge would not be useful to identify any of the parameters, but it would be possible to achieve the identification from the second equation by exploiting the orthogonality condition:
      \itemize{
        \item \eqn{\epsilon_{it} | x_{it}, w_{it-1}, x_{it-1}, m_{it-1},...,x_{i1},w_{i1},m_{i1}) = 0}
      }
    with an instrumental variable version of Robinson (1988)'s estimator.
  }


%%%%%%%%% VALUE %%%%%%%%%%%
  \value{
    The output of the function \code{prodestROB} is a member of the \code{S3} class \pkg{prod}. More precisely, is a list (of length 3) containing the following elements:

      \code{Model}, a list containing:
      \itemize{
        \item \code{method:} a string describing the method ('ROB-IV').
        \item \code{elapsed.time:} time elapsed during the estimation.
        \item \code{opt.outcome:} optimization outcome.
      }

    \code{Data}, a list containing:
      \itemize{
        \item \code{Y:} the vector of value added log output.
        \item \code{free:} the vector/matrix/dataframe of log free variables.
        \item \code{state:} the vector/matrix/dataframe of log state variables.
        \item \code{proxy:} the vector/matrix/dataframe of log proxy variables.
        \item \code{control:} the vector/matrix/dataframe of log control variables.
        \item \code{idvar:} the vector/matrix/dataframe identifying individual panels.
        \item \code{timevar:} the vector/matrix/dataframe identifying time.
      }

    \code{Estimates}, a list containing:
      \itemize{
        \item \code{pars:} the vector of estimated coefficients.
        \item \code{std.errors:} the vector of bootstrapped standard errors.
      }

    Members of class \code{prod} have an \code{omega} method returning a numeric object with the estimated productivity - that is: \eqn{\omega_{it} = y_{it} - (\alpha + w_{it}\beta + k_{it}\gamma)}.
    \code{FSres} method returns a numeric object with the residuals of the first stage regression, while \code{summary}, \code{show} and \code{coef} methods are implemented and work as usual.
  }
%%%%%%%%% AUTHOR %%%%%%%%%%%
  \author{
    Gabriele Rovigatti
  }
%%%%%%%%% REFERENCES %%%%%%%%%%%
  \references{
    Ackerberg, D., K. Caves, and G. Frazer (2015).
    "Identification Properties of Recent Production Function Estimators."
    Econometrica 83(6): 2411-2451.

    Robinson, P. M. (1988).
    "Root-N-consistent semiparametric regression."
    Econometrica: Journal of the Econometric Society, 931-954.

    Wooldridge, J M (2009).
    "On estimating firm-level production functions using proxy variables to control for unobservables."
    Economics Letters, 104, 112-114.
  }
%%%%%%%%% EXAMPLES %%%%%%%%%%%
  \examples{
    data("chilean")

    # we fit a model with two free (skilled and unskilled), one state (capital)
    # and one proxy variable (materials)

    ROB.IV.fit <- prodestROB(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2),
                                   chilean$sX, chilean$pX, chilean$idvar, chilean$timevar)

    # show results
    ROB.IV.fit

    \donttest{
      # estimate a panel dataset - DGP1, various measurement errors - and run the estimation
      sim <- panelSim()

      ROB.IV.sim1 <- prodestROB(sim$Y, sim$fX, sim$sX, sim$pX1, sim$idvar, sim$timevar)
      ROB.IV.sim2 <- prodestROB(sim$Y, sim$fX, sim$sX, sim$pX2, sim$idvar, sim$timevar)
      ROB.IV.sim3 <- prodestROB(sim$Y, sim$fX, sim$sX, sim$pX3, sim$idvar, sim$timevar)
      ROB.IV.sim4 <- prodestROB(sim$Y, sim$fX, sim$sX, sim$pX4, sim$idvar, sim$timevar)

      # show results in .tex tabular format
      printProd(list(ROB.IV.sim1, ROB.IV.sim2, ROB.IV.sim3, ROB.IV.sim4),
                parnames = c('Free','State'))
    }
  }

\name{block.boot.resample}
\alias{block.boot.resample}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Cluster Bootstrap Resampling}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Generate row indices for an individual-level block bootstrap of panel data.
  All observations belonging to a sampled individual are kept together.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  block.boot.resample(idvar, R)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  \item{idvar}{
    One-column vector, matrix or data frame of panel identifiers. Numeric,
    character and factor labels are accepted.
  }
  \item{R}{
    Positive integer giving the number of bootstrap samples.
  }
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  For every replication, the function samples the distinct panel identifiers
  with replacement. It returns a list of one-column integer matrices. Matrix
  values are row positions in the original data, while row names are newly
  generated sequential identifiers for the resampled panels. Consequently, a
  source panel may be selected more than once without merging its copies.
}
%%%%%%%%% VALUE %%%%%%%%%
\value{
  A list of length \code{R}. Each element is a one-column integer matrix of
  resampled row positions with bootstrap panel identifiers stored as row names.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{finalACF}
\alias{finalACF}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{ACF estimation routine}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  \code{finalACF} is the function linking the data cleaning part of the routine with the final function to be bootstrapped.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  finalACF(ind, data, fnum, snum, cnum, opt, theta0, boot = FALSE)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  %
  \item{ind}{
    Vector of indices to reshuffle the data.
  }
  %
  \item{data}{
    \code{data.frame} with the data to perform the estimation on.
  }
  %
  \item{fnum}{
    Number of free variables.
  }
  %
  \item{snum}{
    Number of state variables.
  }
  %
  \item{cnum}{
    Number of control variables.
  }
  %
  \item{opt}{
    String with the optimizer.
  }
  %
  \item{theta0}{
    Vector of starting points.
  }
  %
  \item{boot}{
    Binary indicator for the estimation routine being the baseline estimation (\code{boot = FALSE}, the default) or a bootstrap repetition.
  }
  %
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  \code{finalACF()} accepts at least 7 inputs: a vector of reshuffled indices, the \code{data.frame} with the data, the number of free, state and control variables, the starting points and the optimizer. It collects the results of \code{gACF()} function - baseline and bootstrapped - calculates the standard errors and stores all in a \code{prod} object.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{printProd}
\alias{printProd}
%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%
\title{
  Print output - prod objects
}
%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%
\description{
  The \code{printProd()} function accepts a \code{list} of \code{prod} class objects and returns a screen printed tabular in lateX format of the results.
}
%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%
\usage{
  printProd(mods, modnames = NULL, parnames = NULL, outfile = NULL,
            ptime = FALSE, nboot = FALSE, screen = FALSE)
}
%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%
\arguments{
  \item{mods}{
    a \code{list} of \code{prod} objects.}
  %
  \item{modnames}{
    an optional vector of model names. By default, model names are the \code{@ModelMethod} values in \code{prod} objects.}
  %
  \item{parnames}{
    an optional vector of parameter names. By default, parameter names are the \code{names()} vector of \code{@Estimatespars} in \code{prod} objects.}
  %
  \item{outfile}{
    optional string with the path and directory to store a text file (.txt, .tex, etc. depending on the specified extension) with the tabular. By default \code{outfile = NULL}.}
  %
  \item{ptime}{
    add a row showing the computational time. By default \code{ptime = FALSE}.}
  %
  \item{nboot}{
    add a row showing the number of bootstrap repetitions. By default \code{nboot = FALSE}.}
  %
  \item{screen}{
    print the table on-screen without teX format. By default \code{screen = FALSE}.}
  %
}
%%%%%%%%% VALUE %%%%%%%%%%%
  \value{
The output of the function \code{printProd} is either a screen printed tabular in lateX format of \code{prod} object results or a text file tabular in lateX format of \code{prod} object results.
  }
%%%%%%%%% AUTHOR %%%%%%%%%%%
  \author{
    Gabriele Rovigatti
  }
%%%%%%%%% EXAMPLES %%%%%%%%%%%
  \examples{
    data("chilean")

    # run various models
    WRDGfit <- prodestWRDG_GMM(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2),
                          chilean$sX, chilean$pX, chilean$idvar, chilean$timevar)
    OPfit <- prodestOP(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2), chilean$sX,
                        chilean$pX, chilean$idvar, chilean$timevar)

    # show the output in latex - tabular format
    printProd(list(OPfit, WRDGfit), modnames = c('Olley-Pakes', 'Wooldridge'),
              parnames = c('bunsk', 'bsk', 'bk'))
    # show the output on-screen - no teX format
    printProd(list(OPfit, WRDGfit), modnames = c('Olley-Pakes', 'Wooldridge'),
              parnames = c('bunsk', 'bsk', 'bk'), screen = TRUE)
  }

\name{gACF}
\alias{gACF}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{ACF Second Stage - GMM estimation}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  \code{gACF} returns the second stage parameters estimates of ACF models. It is part of the \code{prodestACF()} routine.
}
%%%%%%%%% USAGE %%%%%%%%%%%
  \usage{
    gACF(theta, mZ, mW, mX, mlX, vphi, vlag.phi)
  }
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
  \arguments{
    %
    \item{theta}{
      Vector of parameters to be estimated.
    }
    %
    \item{mZ}{
      Matrix of instruments.
    }
    %
    \item{mW}{
      Weighting matrix.
    }
    %
    \item{mX}{
      Matrix of regressors.
    }
    %
    \item{mlX}{
      matrix of lagged regressors.
    }
    %
    \item{vphi}{
      Vector of fitted polynomial.
    }
    %
    \item{vlag.phi}{
      Lagged vector of fitted polynomial.
    }
    %
  }
%%%%%%%%% DETAILS %%%%%%%%%
  \details{
    \code{gACF()} estimates the second stage of ACF routine. It accepts 7 inputs, generates and optimizes over the group of moment functions \eqn{E(\xi_{it} Z^k_{it})}{E(xi_it Z_it^k)}.
  }
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{checkMD}
\alias{checkMD}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Change dummy input to dummy matrix}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Function to transform all input to a matrix. In addition, it checks whether all elements of the input are either 0 or 1.
}
%%%%%%%%% USAGE %%%%%%%%%%%
  \usage{
    checkMD(input)
  }
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
  \arguments{
    %
    \item{input}{
      An \code{R} object. Can be a matrix/dataframe/vector/scalar.
    }
    %
  }
%%%%%%%%% DETAILS %%%%%%%%%
  \details{
    \code{checkMD()} accepts one input and - if \code{input} is a matrix - returns it without column names, otherwise transforms it into a matrix and returns it without column names. In case any of the elements of input are different from 0 or 1, it stops the routine and throws an error.
  }
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{show}
\alias{show,prod-method}
\alias{show}
\docType{methods}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Print a table with parameter estimates}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  This method allows the user to print a table with the parameter estimates of an \code{S4} \link{prod} object.
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{
  show(object)
}
%%%%%%%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%%%%
\arguments{
  %
  \item{object}{
    object of class \link{prod}.
  }
  %
}
%%%%%%%%%%%%%%%% DETAILS %%%%%%%%%%%%%%%%%%%
\details{
  \code{show} accepts an \code{S4} \link{prod} object and prints a table with estimated parameters.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{omega}
\alias{omega,prod-method}
\alias{omega}
\docType{methods}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Generate the omega estimates}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  This method provides the way to estimate the omega residuals from a \link{prod} \code{S4} object - estimates from \link{prodestOP}, \link{prodestLP}, \link{prodestACF}, \link{prodestWRDG} and \link{prodestWRDG_GMM} - defined in the \code{prodest} package
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{
  omega(object)
}
%%%%%%%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%%%%
\arguments{
  %
  \item{object}{
    object of class \link{prod}.
  }
  %
}
%%%%%%%%%%%%%%%% DETAILS %%%%%%%%%%%%%%%%%%%
\details{
  \code{omega} accepts an \code{S4} \link{prod} object and returns a vector of omega estimates.
}
%%%%%%%%%%%%%%%% VALUE %%%%%%%%%%%%%%%%%%%
\value{
  \itemize{
    \item A vector of productivity estimates - omega.
  }
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{prodestACF}
\alias{prodestACF}
%%%%%%%%% TITLE %%%%%%%%%%%
\title{
  Estimate productivity - Ackerberg-Caves-Frazer correction
}
%%%%%%%%% DESCRIPTION %%%%%%%%%%%
\description{
  The \code{prodestACF()} function accepts at least 6 objects (id, time, output, free, state and proxy variables), and returns a \code{prod} object of class \code{S3} with three elements: (i) a list of model-related objects, (ii) a list with the data used in the estimation and estimated vectors of first-stage residuals, and (iii) a list with the estimated parameters and their bootstrapped standard errors .
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  prodestACF(Y, fX, sX, pX, idvar, timevar, R = 20, cX = NULL,
            opt = 'optim', theta0 = NULL, cluster = NULL)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  \item{Y      }{
    the vector of value added log output.}
  %
  \item{fX     }{
    the vector/matrix/dataframe of log free variables.}
  %
  \item{sX     }{
    the vector/matrix/dataframe of log state variables.}
  %
  \item{pX    }{
    the vector/matrix/dataframe of log proxy variables.}
  %
  \item{cX     }{
    the vector/matrix/dataframe of control variables to be estimated as production-function regressors. Controls enter both estimation stages; their lagged values also enter the second-stage instrument matrix. By default \code{cX = NULL}.}
  %
  \item{idvar  }{
    a one-column vector/matrix/dataframe identifying individual panels; numeric, character and factor labels are accepted.}
  %
  \item{timevar}{
    the vector/matrix/dataframe identifying time.}
  %
  \item{R      }{
    the number of block bootstrap repetitions to be performed in the standard error estimation. By default \code{R = 20}.}
  %
  \item{opt    }{
    a string with the optimization algorithm to be used during the estimation. By default \code{opt = 'optim'}.}
  %
  \item{theta0 }{
    a vector with the second stage optimization starting points. By default \code{theta0 = NULL} and the optimization is run starting from the first stage estimated parameters + \eqn{N(0,0.01)} noise.}
  %
  \item{cluster}{
    an object of class \code{"SOCKcluster"} or \code{"cluster"}. By default \code{cluster = NULL}.}
  %
}
%%%%%%%%% DETAILS %%%%%%%%%%%
  \details{
    Consider a Cobb-Douglas production technology for firm \eqn{i} at time \eqn{t}
    \itemize{
      \item \eqn{y_{it} = \alpha + w_{it}\beta + k_{it}\gamma + \omega_{it} + \epsilon_{it}}
    }
    where \eqn{y_{it}} is the (log) output, \eqn{w_{it}} is a \eqn{1 \times J}{1 x J} vector of (log) free variables, \eqn{k_{it}} is a \eqn{1 \times K}{1 x K} vector of state variables, and \eqn{\epsilon_{it}} is a normally distributed idiosyncratic error term.
    The unobserved technical efficiency parameter \eqn{\omega_{it}} evolves according to a first-order Markov process:
      \itemize{
        \item \eqn{\omega_{it} = E(\omega_{it} | \omega_{it-1}) + u_{it} = g(\omega_{it-1}) + u_{it}}
      }
    and \eqn{u_{it}} is a random shock component assumed to be uncorrelated with the technical efficiency, the state variables in \eqn{k_{it}} and the lagged free variables \eqn{w_{it-1}}.
    ACF propose an estimation algorithm alternative to OP and LP procedures claiming that the labour demand and the control function are partially collinear.
    It is based on the following set of assumptions:
      \itemize{
        \item a) \eqn{p_{it} = p(k_{it} , l_{it} , \omega_{it})} is the proxy variable policy function;
        \item b) \eqn{p_{it}} is strictly monotone in \eqn{\omega_{it}};
        \item c) \eqn{\omega_{it}} is scalar unobservable in \eqn{p_{it} = m(.)} ;
        \item d) The state variable are decided at time t-1. The less variable labor input, \eqn{l_{it}}, is chosen at t-b, where \eqn{0 < b < 1}. The free variables, \eqn{w_{it}}, are chosen in t when the firm productivity shock is realized.
      }
      Under this set of assumptions, the first stage is meant to remove the shock \eqn{\epsilon_{it}} from the the output, \eqn{y_{it}}. As in the OP/LP case, the inverted policy function replaces the productivity term \eqn{\omega_{it}} in the production function:
      \itemize{
        \item \eqn{y_{it} = k_{it}\gamma + w_{it}\beta + l_{it}\mu + h(p_{it} , k_{it} ,w_{it} , l_{it}) +  \epsilon_{it}}
      }
  which is estimated by a non-parametric approach - First Stage.
  Exploiting the Markovian nature of the productivity process one can use assumption d) in order to set up the relevant moment conditions and estimate the production function parameters - Second stage.
  }
%%%%%%%%% VALUE %%%%%%%%%%%
  \value{
    The output of the function \code{prodestACF} is a member of the \code{S3} class \pkg{prod}. More precisely, is a list (of length 3) containing the following elements:

    \code{Model}, a list with elements:
    \itemize{
      \item \code{method:} a string describing the method ('ACF').
      \item \code{boot.repetitions:} the number of bootstrap repetitions used for standard errors' computation.
      \item \code{elapsed.time:} time elapsed during the estimation.
      \item \code{theta0:} numeric object with the optimization starting points - second stage.
      \item \code{opt:} string with the optimization routine used - 'optim', 'solnp' or 'DEoptim'.
      \item \code{opt.outcome:} optimization outcome.
      \item \code{FSbetas:} first stage estimated parameters.
    }

    \code{Data}, a list with elements:
    \itemize{
      \item \code{Y:} the vector of value added log output.
      \item \code{free:} the vector/matrix/dataframe of log free variables.
      \item \code{state:} the vector/matrix/dataframe of log state variables.
      \item \code{proxy:} the vector/matrix/dataframe of log proxy variables.
      \item \code{control:} the vector/matrix/dataframe of log control variables.
      \item \code{idvar:} the vector/matrix/dataframe identifying individual panels.
      \item \code{timevar:} the vector/matrix/dataframe identifying time.
      \item \code{FSresiduals:} numeric object with the residuals of the first stage.
    }

    \code{Estimates}, a list with elements:
    \itemize{
      \item \code{pars:} the vector of estimated coefficients.
      \item \code{std.errors:} the vector of bootstrapped standard errors.
    }

    Members of class \code{prod} have an \code{omega} method returning a numeric object with the estimated productivity - that is: \eqn{\omega_{it} = y_{it} - (\alpha + w_{it}\beta + k_{it}\gamma)}.
    \code{FSres} method returns a numeric object with the residuals of the first stage regression, while \code{summary}, \code{show} and \code{coef} methods are implemented and work as usual.
  }
%%%%%%%%% AUTHOR %%%%%%%%%%%
  \author{
    Gabriele Rovigatti
  }
%%%%%%%%% REFERENCES %%%%%%%%%%%
  \references{
    Ackerberg, D., Caves, K. and Frazer, G. (2015).
    "Identification properties of recent production function estimators."
    Econometrica, 83(6), 2411-2451.
  }
%%%%%%%%% EXAMPLES %%%%%%%%%%%
  \examples{

    require(prodest)

    ## Source: Instituto Nacional de Estadisticas de Chile (ENIA).

    data(chilean)

    # we fit a model with two free (skilled and unskilled), one state (capital)
    # and one proxy variable (materials)

    ACF.fit <- prodestACF(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2), chilean$sX,
                          chilean$pX, chilean$idvar, chilean$timevar,
                          theta0 = c(.5,.5,.5), R = 5)
    \donttest{
      set.seed(154673)
      ACF.fit.solnp <- prodestACF(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2), chilean$sX,
                            chilean$pX, chilean$idvar, chilean$timevar,
                            theta0 = c(.5,.5,.5), opt = 'solnp')

      # run the same regression in parallel
      # nCores <- as.numeric(Sys.getenv("NUMBER_OF_PROCESSORS")) # Windows systems
      nCores <- 3
      cl <- makeCluster(getOption("cl.cores", nCores - 1))
      set.seed(154673)
      ACF.fit.par <- prodestACF(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2), chilean$sX,
                                chilean$pX, chilean$idvar, chilean$timevar,
                                theta0 = c(.5,.5,.5), cluster = cl)
      stopCluster(cl)

      # show results
      coef(ACF.fit)
      coef(ACF.fit.solnp)

       # show results in .tex tabular format
       printProd(list(ACF.fit, ACF.fit.solnp))
    }
  }

\name{coef}
\alias{coef,prod-method}
\alias{coef}
\docType{methods}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Print the estimated parameters}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  This method provides the way to extract and print the estimated parameters from a \link{prod} \code{S4} object - estimates from \link{prodestOP}, \link{prodestLP}, \link{prodestACF}, \link{prodestWRDG} and \link{prodestWRDG_GMM} - defined in the \code{prodest} package
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{
  coef(object,...)
}
%%%%%%%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%%%%
\arguments{
  %
  \item{object}{
    object of class \link{prod}.
  }
  %
  \item{...}{
    Additional arguments.
  }
  %
}
%%%%%%%%%%%%%%%% DETAILS %%%%%%%%%%%%%%%%%%%
\details{
  \code{coef} accepts an \code{S4} \link{prod} object and prints the vector of estimated parameters.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{prodestOP}
\alias{prodestOP}
\title{
	Estimate productivity - Olley-Pakes method
}
\description{
The \code{prodestOP()} function accepts at least 6 objects (id, time, output, free, state and proxy variables), and returns a \code{prod} object of class \code{S4} with three elements: (i) a list of model-related objects, (ii) a list with the data used in the estimation and estimated vectors of first-stage residuals, and (iii) a list with the estimated parameters and their bootstrapped standard errors .
}
\usage{
  prodestOP(Y, fX, sX, pX, idvar, timevar, R = 20, cX = NULL,
            opt = 'optim', theta0 = NULL, cluster = NULL, tol = 1e-100, exit = FALSE)
}
\arguments{
  \item{Y      }{
  the vector of value added log output.}
  %
  \item{fX     }{
  the vector/matrix/dataframe of log free variables.}
  %
  \item{sX     }{
  the vector/matrix/dataframe of log state variables.}
  %
  \item{pX    }{
  the vector/matrix/dataframe of log proxy variables.}
  %
  \item{cX     }{
  the vector/matrix/dataframe of control variables. By default \code{cX= NULL}.}
  %
  \item{idvar  }{
  a one-column vector/matrix/dataframe identifying individual panels; numeric, character and factor labels are accepted.}
  %
  \item{timevar}{
  the vector/matrix/dataframe identifying time.}
  %
  \item{R      }{
  the number of block bootstrap repetitions to be performed in the standard error estimation. By default \code{R = 20}.}
  %
  \item{opt    }{
  a string with the optimization algorithm to be used during the estimation. By default \code{opt = 'optim'}.}
  %
  \item{theta0 }{
  a vector with the second stage optimization starting points. By default \code{theta0 = NULL} and the optimization is run starting from the first stage estimated parameters + \eqn{N(\mu=0,\sigma=0.01)} noise.}
  %
  \item{cluster}{
    an object of class \code{"SOCKcluster"} or \code{"cluster"}. By default \code{cluster = NULL}.}
  %
  \item{tol}{
    optimizer tolerance. By default \code{tol = 1e-100}.}
  %
  \item{exit}{
    Indicator for attrition in the data - i.e., if firms exit the market. By default \code{exit = FALSE}; if \code{exit = TRUE}, an indicator function for firms whose last appearance is before the last observation's date is generated and used in the second stage. The user can even specify an indicator variable/matrix/dataframe with the exit years.}
  %
}
%%%%%%%%% DETAILS %%%%%%%%%%%
\details{
  Consider a Cobb-Douglas production technology for firm \eqn{i} at time \eqn{t}
  \itemize{
    \item \eqn{y_{it} = \alpha + w_{it}\beta + k_{it}\gamma + \omega_{it} + \epsilon_{it}}
  }
  where \eqn{y_{it}} is the (log) output, \eqn{w_{it}} is a \eqn{1 \times J}{1 x J} vector of (log) free variables, \eqn{k_{it}} is a \eqn{1 \times K}{1 x K} vector of state variables, and \eqn{\epsilon_{it}} is a normally distributed idiosyncratic error term.
  The unobserved technical efficiency parameter \eqn{\omega_{it}} evolves according to a first-order Markov process:
  \itemize{
    \item \eqn{\omega_{it} = E(\omega_{it} | \omega_{it-1}) + u_{it} = g(\omega_{it-1}) + u_{it}}
  }
  and \eqn{u_{it}} is a random shock component assumed to be uncorrelated with the technical efficiency, the state variables in \eqn{k_{it}} and the lagged free variables \eqn{w_{it-1}}.
  The OP method relies on the following set of assumptions:
  \itemize{
    \item a) \eqn{i_{it} = i(k_{it},\omega_{it})} - investments are a function of both the state variable and the technical efficiency parameter;
    \item b) \eqn{i_{it}} is strictly monotone in \eqn{\omega_{it}};
    \item c) \eqn{\omega_{it}} is scalar unobservable in \eqn{i_{it} = i(.)} ;
    \item d) the levels of \eqn{i_{it}} and \eqn{k_{it}} are decided at time \eqn{t-1}; the level of the free variable, \eqn{w_{it}}, is decided after the shock \eqn{u_{it}} realizes.
  }
  Assumptions a)-d) ensure the invertibility of \eqn{i_{it}} in \eqn{\omega_{it}} and lead to the partially identified model:
  \itemize{
    \item \eqn{y_{it} = \alpha + w_{it}\beta + k_{it}\gamma + h(i_{it}, k_{it}) + \epsilon_{it} = \alpha + w_{it}\beta  + \phi(i_{it}, k_{it}) + \epsilon_{it} }
  }
  which is estimated by a non-parametric approach - First Stage.
  Exploiting the Markovian nature of the productivity process one can use assumption d) in order to set up the relevant moment conditions and estimate the production function parameters - Second stage.
  Exploiting the residual \eqn{e_{it}} of:
  \itemize{
    \item \eqn{y_{it} - w_{it}\hat{\beta} = \alpha + k_{it}\gamma + g(\omega_{it-1}, \chi_{it}) + \epsilon_{it} }
  }
  and \eqn{g(.)} is typically left unspecified and approximated by a \eqn{n^{th}} order polynomial and \eqn{\chi_{it}} is an indicator function for the attrition in the market.
}
%%%%%%%%% VALUE %%%%%%%%%%%
\value{
    The output of the function \code{prodestOP} is a member of the \code{S3} class \pkg{prod}. More precisely, is a list (of length 3) containing the following elements:

    \code{Model}, a list containing:
    \itemize{
      \item \code{method:} a string describing the method ('OP').
      \item \code{boot.repetitions:} the number of bootstrap repetitions used for standard errors' computation.
      \item \code{elapsed.time:} time elapsed during the estimation.
      \item \code{theta0:} numeric object with the optimization starting points - second stage.
      \item \code{opt:} string with the optimization routine used - 'optim', 'solnp' or 'DEoptim'.
      \item \code{opt.outcome:} optimization outcome.
      \item \code{FSbetas:} first stage estimated parameters.
    }

    \code{Data}, a list containing:
    \itemize{
      \item \code{Y:} the vector of value added log output.
      \item \code{free:} the vector/matrix/dataframe of log free variables.
      \item \code{state:} the vector/matrix/dataframe of log state variables.
      \item \code{proxy:} the vector/matrix/dataframe of log proxy variables.
      \item \code{control:} the vector/matrix/dataframe of log control variables.
      \item \code{idvar:} the vector/matrix/dataframe identifying individual panels.
      \item \code{timevar:} the vector/matrix/dataframe identifying time.
      \item \code{FSresiduals:} numeric object with the residuals of the first stage.
    }

    \code{Estimates}, a list containing:
    \itemize{
      \item \code{pars:} the vector of estimated coefficients.
      \item \code{std.errors:} the vector of bootstrapped standard errors.
    }

    Members of class \code{prod} have an \code{omega} method returning a numeric object with the estimated productivity - that is: \eqn{\omega_{it} = y_{it} - (\alpha + w_{it}\beta + k_{it}\gamma)}.
    \code{FSres} method returns a numeric object with the residuals of the first stage regression, while \code{summary}, \code{show} and \code{coef} methods are implemented and work as usual.
}
%%%%%%%%% AUTHOR %%%%%%%%%%%
\author{Gabriele Rovigatti}
%%%%%%%%% REFERENCES %%%%%%%%%%%
\references{
  Olley, S G and Pakes, A (1996).
  "The dynamics of productivity in the telecommunications equipment industry."
  Econometrica, 64(6), 1263-1297.
}
%%%%%%%%% EXAMPLES %%%%%%%%%%%
\examples{
    require(prodest)

    ## Source: Instituto Nacional de Estadisticas de Chile (ENIA).

    data(chilean)

    # we fit a model with two free (skilled and unskilled), one state (capital)
    # and one proxy variable (investment)

    OP.fit <- prodestOP(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2), chilean$sX,
                        chilean$inv, chilean$idvar, chilean$timevar)
    OP.fit.solnp <- prodestOP(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2),
                              chilean$sX, chilean$inv, chilean$idvar,
                              chilean$timevar, opt='solnp')
    OP.fit.attrition <- prodestOP(chilean$Y, fX = cbind(chilean$fX1, chilean$fX2),
                                chilean$sX, chilean$inv, chilean$idvar,
                                chilean$timevar, exit = TRUE)

    # show results
    summary(OP.fit)
    summary(OP.fit.solnp)

    # show results in .tex tabular format
     printProd(list(OP.fit, OP.fit.solnp, OP.fit.attrition))
}

\name{weightM}
\alias{weightM}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Generate optimal GMM weighting matrix}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  In a Wooldridge estimation setting, i.e., in a system GMM framework, this function returns the optimal weighting matrix or the variance-covariance matrix given 1st or 2nd stage estimation results.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  weightM(Y, X1, X2, Z1, Z2, betas, numR, SE = FALSE)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  %
  \item{Y}{
    Vector of log(value added output).
  }
  %
  \item{X1}{
    Matrix of regressors for the first equation.
  }
  %
  \item{X2}{
    Matrix of regressors for the second equation.
  }
  %
  \item{Z1}{
    Matrix of instruments for the first equation.
  }
  %
  \item{Z2}{
    Matrix of instruments for the second equation.
  }
  %
  \item{betas}{
    Vector of first/second stage parameter estimates.
  }
  %
  \item{numR}{
    Number of state + number of free + number of control variables (i.e., number of constrained parameters).
  }
  %
  \item{SE}{
    Binary indicator for first (\code{SE == FALSE}, the default) or second stage.
  }
  %
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  \code{weightM()} accepts at least 7 inputs: Y, X1, X2, Z1, Z2, betas and numR. With these, computes the optimal weighting matrix in a system GMM framework, i.e., \eqn{W^* = \sigma Z^\top Z}{W* = sigma Z'Z}. If it is called during the first stage, it returns W*, otherwise will return an estimate of the parameters' standard errors, i.e., the square root of the diagonal of the variance-covariance matrix \eqn{N^{-1} [(X^\top Z) W^* (Z^\top X)]^{-1}}{1/N [(X'Z) W* (Z'X)]^-1}.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{panelSim}
\alias{panelSim}
\title{
  Simulate Panel dataset
}
%%%%%%%%% DESCRIPTION %%%%%%%%%%%
\description{
  \code{panelSim()} produces a N*T balanced panel dataset of firms' production. In particular, it returns a \code{data.frame} with free, state and proxy variables aimed at performing Monte Carlo simulations on productivity-related models.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  panelSim(N = 1000, T = 100, alphaL = .6, alphaK = .4, DGP = 1,
           rho = .7, sigeps = .1, sigomg = .3, rholnw = .3)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  \item{N}{
    the number of firms. By default \code{N=1000}}
  %
  \item{T}{
    the total time span to be simulated. Only a fraction (the last 10\% of observations) will be returned. By default \code{T=100}}
  %
  \item{alphaL}{
    the parameter of the free variable. By default \code{alphaL=.6}}
  %
  \item{alphaK}{
    the parameter of the state variable. By default \code{alphaK=.4}}
  %
  \item{DGP}{
    Type of DGP; accepts 1, 2 or 3. They differ in terms of shock to wages (0 or 0.1), \eqn{\Delta} (0 or 0.5) and shock to labor (0 or 0.37). See \eqn{details}. By default \code{DGP=1}.}
  %
  \item{rho}{
    the AR(1) coefficient for omega. By default \code{rho=0.7}}
  %
  \item{sigeps}{
    the standard deviation of epsilon. See \eqn{details}. By default \code{sigeps = .1}.}
  %
  \item{sigomg}{
    the standard deviation of the innovation to productivity \eqn{\omega}. By default \code{sigomg = .3}.}
  %
  \item{rholnw}{
    AR(1) coefficient for log(wage). By default \code{rholnw=.3}.}
  %
}
%%%%%%%%% DETAILS %%%%%%%%%%%
\details{
  \code{panelSim()} is the R implementation of the DGP written by Ackerberg, Caves and Frazer (2015).
}
%%%%%%%%% VALUE %%%%%%%%%%%
\value{
  A \code{data.frame} with nine variables:
  \item{idvar}{
    ID codes from 1 to \eqn{N} (by default \eqn{N = 1000}).
  }
  \item{timevar}{
    Time variable ranging from 1 to \code{round(T * 0.1)} (by default
    \eqn{T = 100} and \code{max(timevar) = 10}).
  }
  \item{Y}{
    Log output value-added variable.
  }
  \item{sX}{
    Log state variable.
  }
  \item{fX}{
    Log free variable.
  }
  \item{pX1}{
    Log proxy variable with no measurement error.
  }
  \item{pX2}{
    Log proxy variable with measurement-error standard deviation
    \eqn{\sigma = 0.1}.
  }
  \item{pX3}{
    Log proxy variable with measurement-error standard deviation
    \eqn{\sigma = 0.2}.
  }
  \item{pX4}{
    Log proxy variable with measurement-error standard deviation
    \eqn{\sigma = 0.5}.
  }
}
%%%%%%%%% AUTHOR %%%%%%%%%%%
\author{Gabriele Rovigatti}
%%%%%%%%% REFERENCES %%%%%%%%%%%
\references{
  Ackerberg, D., Caves, K. and Frazer, G. (2015).
  "Identification properties of recent production function estimators."
  Econometrica, 83(6), 2411-2451.
}
%%%%%%%%% EXAMPLES %%%%%%%%%%%
\examples{

  require(prodest)

  ## Simulate a dataset with 100 firms (T = 50).
  ## \code{Panelsim()} delivers the last 10\% of usable time per panel.

  panel.data <- panelSim(N = 100, T = 50)
  attach(panel.data)

  ## Estimate various models
  ACF.fit <- prodestACF(Y, fX, sX, pX2, idvar, timevar, theta0 = c(.5,.5))
  \donttest{
    LP.fit <- prodestLP(Y, fX, sX, pX2, idvar, timevar)
    WRDG.fit <- prodestWRDG(Y, fX, sX, pX3, idvar, timevar)

    ## print results in lateX tabular format
    printProd(list(LP.fit, ACF.fit, WRDG.fit))
  }
}

\name{finalOPLP}
\alias{finalOPLP}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{OP and LP estimation routine}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  \code{finalOPLP} is the function linking the data cleaning part of the routine with the final function to be bootstrapped.
}
%%%%%%%%% USAGE %%%%%%%%%%%
\usage{
  finalOPLP(ind, data, fnum, snum, cnum, opt, theta0, boot, tol, att)
}
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
\arguments{
  %
  \item{ind}{
    Vector of indices to reshuffle the data.
  }
  %
  \item{data}{
    \code{data.frame} with the data to perform the estimation on.
  }
  %
  \item{fnum}{
    Number of free variables.
  }
  %
  \item{snum}{
    Number of state variables.
  }
  %
  \item{cnum}{
    Number of control variables.
  }
  %
  \item{opt}{
    String with the optimizer.
  }
  %
  \item{theta0}{
    Vector of starting points.
  }
  %
  \item{boot}{
    Binary indicator for the estimation routine being the baseline estimation or a bootstrap repetition.
  }
  %
  \item{tol}{
    Optimization tolerance set.
  }
  %
  \item{att}{
    Indicator for attrition in the data - i.e., if  firms exit the market.
  }
  %
}
%%%%%%%%% DETAILS %%%%%%%%%
\details{
  \code{finalOPLP()} accepts at 9 inputs: a vector of reshuffled indices, the \code{data.frame} with the data, the number of free, state and control variables, the starting points, the optimizer, an indicator for bootstrapped repetitions and the optimization tolerance. It collects the results of \code{gACF()} function - baseline and bootstrapped - calculates the standard errors and stores all in a \code{prod} object.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{lagPanel}
\alias{lagPanel}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Generate lagged input variables}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  Function to generate lagged variables in a panel.
}
%%%%%%%%% USAGE %%%%%%%%%%%
  \usage{
    lagPanel(idvar, timevar, value)
  }
%%%%%%%%% ARGUMENTS %%%%%%%%%%%
  \arguments{
    %
    \item{idvar}{
      vector of panel identifiers.
    }
    %
    \item{timevar}{
      vector of time identifiers.
    }
    %
     \item{value}{
      numeric vector, matrix or data frame containing the variable or variables to be lagged.
    }
    %
  }
%%%%%%%%% DETAILS %%%%%%%%%
  \details{
    \code{lagPanel()} accepts panel identifiers, numeric time identifiers and one or more variables to be lagged. It returns a vector for vector input and a matrix for matrix or data-frame input. Observations without a corresponding value at \eqn{t-1} are returned as \code{NA}. The combination of \code{idvar} and \code{timevar} must uniquely identify observations.
  }
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}

\name{FSres}
\alias{FSres,prod-method}
\alias{FSres}
\docType{methods}
%%%%%%%%%%%%%%%% TITLE %%%%%%%%%%%%%%%%%%%
\title{Generate the vector of the first stage residuals}
%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%
\description{
  This method provides the way to estimate the first stage residuals from a \link{prod} \code{S4} object - estimates from \link{prodestOP}, \link{prodestLP}, \link{prodestACF}, \link{prodestWRDG} and \link{prodestWRDG_GMM} - defined in the \code{prodest} package
}
%%%%%%%%%%%%%%%% USAGE %%%%%%%%%%%%%%%%%%%
\usage{
  FSres(object)
}
%%%%%%%%%%%%%%%% ARGUMENTS %%%%%%%%%%%%%%%%%%%
\arguments{
  %
  \item{object}{
    object of class \link{prod}.
  }
  %
}
%%%%%%%%%%%%%%%% DETAILS %%%%%%%%%%%%%%%%%%%
\details{
  \code{FSres} accepts an \code{S4} \link{prod} object and returns the vector of firs stage residuals.
}
%%%%%%%%%%%%%%%% AUTHOR %%%%%%%%%%%%%%%%%%%
\author{Gabriele Rovigatti}
